
checkwindow <- function(x){
  # This function makes sure that the new connection meshes with previosly established connections
  #  specifically:
  #     all cellsizes should be factors or multiples of all other cellsizes
  #     the upper left corner of all connections should match
  #     the extent of all connections should match as closely as possible given the cellsize.
  #       the first connection established is the refrence extent to which all others are compared.
  # Arguments
  #  x : details on the new connection (xll, yll, nrow, ncol, cellsize, set)
  # 
  # Value a list with two items:
  #   r = logical, TRUE indicates that the new extent is valid, FALSE indicates that it isn't.
  #   msg = if r is FALSE msg indicates why it failed.
  
  if(!x$set) return(list(r=TRUE, msg=NA)) # if it's not set yet than it can't conflict
  
  rw <- .status$referencewindow
  if( is.na(rw$cellsize)) return(list(r=TRUE, msg=NA)) # if there aren't any previous connections it can't conflict
  
  # Check that the upper left corner matches 
  xul <- x$xll
  yul <- with(x, yll + nrow * cellsize)
  rxul <- rw$xll
  ryul <- with(rw, yll + nrow * cellsize)

  
  if(!isTRUE(all.equal(xul, rxul))) return(list(r=FALSE, msg="Upper left corner x doesn't match previous connections") )
  if(!isTRUE(all.equal(yul, ryul))) return(list(r=FALSE, msg="Upper left corner y doesn't match previous connections") )
  
  # Check that the extent is proper
  #   the right and bottom edges should either match the edges of the first extent or 
  #      extend beyond them by less than the cellsize.
  # 
  xlr <- x$xll + x$ncol * x$cellsize 
  refxlr <- rw$xll + rw$ncol * rw$cellsize 
  
  tol <- .Machine$double.eps ^ 0.5
  
  diff <- xlr - refxlr
  if(diff < 0 - tol ) return(list(r=FALSE, msg="The right edge of the new window is short of the first connection's extent."))
  if(diff >= x$cellsize) return(list(r=FALSE, msg="The right edge of the connection extends too far beyond the first connection"))
  diff <-  rw$yll - x$yll  # note refrence window y value should be greater (lower bottom = larger extent) 
  if(diff < 0 - tol) return(list(r=FALSE, msg="The bottom edge of the new window is short of the first connection's extent."))
  if(diff >= x$cellsize) return(list(r=FALSE, msg="The bottom edge of the connection extends too far beyond the first connection."))
  
  prior.cellsizes <- .status$connections$cellsize
  
  # Check that this cellsize is compatible
  sv <- prior.cellsizes < x$cellsize
  sv[is.na(prior.cellsizes)] <- FALSE
  if(any(sv)){
  m <- x$cellsize %% prior.cellsizes[sv]
  if(any(m != 0)) return(list(r=FALSE, msg="cellsize isn't a multiple of all prior, smaller cellsizes"))  
  }
  sv <- prior.cellsizes > x$cellsize
  sv[is.na(prior.cellsizes)] <- FALSE
  if(any(sv)){
    m <- prior.cellsizes %% x$cellsize
    if(any(m != 0)) return(list(r=FALSE, msg="cellsize isn't a factor of all prior, larger cellsizes"))  
  }
  
  
  return(list(r=TRUE, msg=NA))
  
}
