.readblock <- function(path, startrow, startcol, nrow, ncol, buffer=0, extent, as.matrix=FALSE, 
                       na.value=-9999, as.integer, desc){ 
  #------------------------------------------------------------------------------------------------#
  # .readblock    Ethan Plunektt     January 2011
  #  This function reads in an arbitrary rectangular block from a grid and optionally an additional
  #    buffer around the block.
  #  Arguments :
  #    path : the path to the grid
  #    startrow, startcol : the row and column of the start of the core of the block
  #    nrow, ncol : the dimensions of the core of the block
  #    buffer : the amount of extra cells to read (beyond the core of the block)
  #    extent : an alternate way of describing the area to be read. extent should be a list object with 
  #      named elements xll, yll, cellsize, nrow,and ncol which define the block to be read.  This
  #      can be a grid object, the result of a call to griddescribe, gridintersect, or crop or
  #      a list the user has assembled in some other manner.  If supplied the list argument supercedes
  #      startcol, startrow, nrow, and ncol.
  #    as.matrix : if TRUE the function will return a matrix instead of a grid object
  #    na.value : this is the value used to pass NA's between R and C code.  On either
  #       end it is converted to native NA formats.  Set it to something other than 
  #       -9999 if you have cells with that value in your grid (that aren't NA).
  #    as.integer : (optional, logical) if supplied the function will assume that the type indicated
  #      by this argument (TRUE indicates integer FALSE indicates real) matches the grid type on
  #      disk and skip checking the type on disk before attempting the write.  This is quicker but 
  #      carries a greater risk of error. 
  #  Changes:
  #     Feb 24 2012, Added as.integer argument
  #     Sept 2013 - changed to .readblock  readblock is now a higher level funciton that handles
  #       mosaiced grids and calls readblockc which handles caching which calls .readblock to 
  #       do the read.  
  #------------------------------------------------------------------------------------------------#
  
  
  verbose <- .status$verbosity >= 3
  
  if(missing(as.integer)){
    type <- desc$type
    if(type == "missing") stop("Grid ", path, " not found.")
    as.integer <- type == "integer"
  }
  
  stopifnot(coincide(desc, getwindow(pane.only=TRUE)))
  w <- desc
  
  
  if(!missing(extent)){
    stopifnot(all(c("xll", "yll", "nrow", "ncol", "cellsize") %in% names(extent)))
    nrow <- extent$nrow
    ncol <- extent$ncol
    cellsize <- extent$cellsize
    xll <- extent$xll
    yll <- extent$yll
    startcol <- x2c(xll + 0.5 * cellsize, list = w)
    startrow <- y2r(yll + 0.5 * cellsize, list = w) -  nrow + 1
  }
  
  
  if(nrow * ncol > .status$blocksizelimit)
    stop("Blocks can't be bigger than ", .status$blocksizelimit, " cells. (", 
         sqrt(.status$blocksizelimit), " cells in each dimension if square)")
  
  
  m <- matrix(0, nrow=nrow + 2*buffer, ncol=ncol+2*buffer)
  # C function declaration:
  #CAPS_API void R_readblock_dbl(int *ret_v, char **grid_name, int *y_start, int *x_start, int *y_count, int *x_count, int *buffer_size, double *missing_value, double *mtx_block) ;
  path <- as.character(path)
  if(!(length(path)==1 && !is.na(path) && !is.null(path))) stop("read block called with NULL or NA path")
  
  if(verbose){
    cat("\n.readblock called with args:\n\t",
        "path = ", path, "\n\t",
        "startrow = ", startrow, "\n\t", 
        "startcol = ", startcol, "\n\t", 
        "nrow = ", nrow, "\n\t", 
        "ncol = ", ncol, "\n\t", 
        "buffer = ", buffer, "\n\t", 
        "na.value = ", na.value, "\n\t", 
        "with a ", nrow(m), " by ", ncol(m), " matrix\n\n"
    )
  }
  
  done <- FALSE
  .status$gsdowntime <- 0  # The amount of time the gridserver has been down
  while(!done){
    st <- system.time({
      if(!as.integer){ # double
        r <- .C("R_readblock_dbl", integer(1), as.character(path), as.integer(startrow), 
                as.integer(startcol), as.integer(nrow), as.integer(ncol), as.integer(buffer), 
                as.double(na.value), as.double(m), PACKAGE="caps_lib")
      } else {  # integer
        r <- .C("R_readblock_int", integer(1), as.character(path), as.integer(startrow), 
                as.integer(startcol), as.integer(nrow), as.integer(ncol), as.integer(buffer), 
                as.integer(na.value), as.integer(m), PACKAGE="caps_lib")
      }  
    })
    .status$gridwait <- .status$gridwait + as.numeric(st[3])
    done <- .donetrying(code=r[[1]], st, reset=TRUE)
  }
  if(r[[1]] < 0){ # if there's an error
    a <- checkgrid(path, test = FALSE)  # check for common problems that can be described
    if(a$problem){
      a$problem <- NULL
      stop("readblock error with ", path," : ", paste(names(a)[unlist(a)], collapse = ", "))
    }  
    # if no identifable problem throw EDI's error
    parseerror(r[[1]], "readblock")
  }
  m <- matrix(r[[length(r)]], nrow=nrow + 2 * buffer, ncol=ncol+2*buffer, byrow=TRUE)
  m[m==na.value] <- NA
  if(as.matrix) return(m)
  
  result <- list(
    m = m,
    xll = w$xll + w$cellsize * ( startcol - 1 - buffer),
    yll = w$yll + w$cellsize * (w$nrow - (startrow - 1) - (nrow(m) -buffer) ) ,
    nrow = nrow(m),
    ncol = ncol(m),
    cellsize = w$cellsize
  )
  
  
  class(result) <- c("grid", class(result))
  return(result)
} # end readblock function definition

