.writeblock <-
function(x, path, startrow, startcol, buffer=0, transparent=TRUE, add=FALSE, as.integer, 
	na.value=-9999){
#--------------------------------------------------------------------------------------------------#
#  .writeblock   Ethan PlunkettJanuary 2011
#  
#  Function to write a matrix into an arbitrary block of grid on disk.   The type of data to be
#    written is determine by the exisiting grid.  If no grid exists than the as.integer flag 
#    controls the type of grid that is created (integer or real).  Setting transparent to TRUE 
#    allows writing of lapping non-rectangular data, such as watersheds; only non-NA data will 
#    be written.  Setting add to TRUE adds the data in x to the existing grid 
#    (skipping cells with NA). add=TRUE takes precidence over transperent.  This is the matrix 
#    method for the writeblock function and thus will be used automatically when writeblock 
#    is called on a matrix object.
#    
#  Arguments:
#    x : a matrix to write
#    path : the path to the grid (can be just the name)
#    startrow, startcol: starting row or col, in terms of cells of the reference grid (if buffer
#is specified this should be the starting row and column for the core (not buffer) of x 
#    buffer : buffer around matrix in cells, these cells will be cropped from the matrix prior to 
#      writing.
#    transparent : if TRUE NA values will not be written to the grid (existing values will persist) 
#    add : if TRUE the value of the matrix will be added to the grid (NA cells will be skipped).
#      add takes precedance over transparent.
#    as.integer : (optional) if provided the function will assume that the grid on disk will 
#      will match the indicated type (integer or real depending on whether the argument is TRUE or
#      FALSE).  Additionally if the grid doesn't exist on disk one will be created
#      of the indicated type.  If missing than the function will always check the grid on disk prior
#      to writing and write of the appropriate type (coersing x to integer if necessary).  If missing
#      and the grid doesn't exist a "real" grid will be created with a warning.
#    na.value : the value used to tranmit NA values to the C function.  Set it to something else
#      if there are valid -9999 cells in your grid.
#--------------------------------------------------------------------------------------------------#
  
  
  if(.status$verbosity >= 3){
    cat("\n.writeblock called with arguments:\n\t", 
        "x= (a ", nrow(x), " by ", ncol(x),  " matrix not printed)", "\n\t",
        "path=", path,  "\n\t",
        "startrow=", startrow,  "\n\t",
        "startcol=", startcol,  "\n\t",
        "buffer=", buffer,  "\n\t",
        "transparent=", transparent,  "\n\t",
        "add=", add,  "\n\t",
        "as.integer=", as.integer,  "\n\t",
        "na.value=", na.value,  "\n\n", 
        sep="")    
  }
  
  if(nrow(x) * ncol(x) > .status$blocksizelimit)
    stop("Blocks can't be bigger than ", .status$blocksizelimit, " cells. (", 
         sqrt(.status$blocksizelimit), " cells in each dimension if square)")
  
  
  if(missing(as.integer)){  # if the user hasn't specified the type check it
    # Note .writeblock should only be called from writeblock so this should never happen
     d <- griddescribe(path, cache = FALSE) 
     if(d$type == "missing"){
       warning("as.integer was not specified and the grid didn't exist on disk. A real (floating point) grid was created by default.")
       as.integer = FALSE
       gridcreate(path, as.integer=as.integer)     
     } else {
       as.integer <- d$type == "integer"
     }
  }
  
  buffer <- as.integer(buffer) # enforce integer buffer
  if(buffer < 0) stop("Buffer cannot be negative")
  if(buffer > 0){
    x <- crop(x, buffer)
  }
  
  x[is.na(x)] <- na.value
  tc <- 0  # transparency code
  if(transparent) tc <- 1
  if(add) tc <- 2

  # Note if the written block extends beyond the edge of the access window the C function handles 
  #  things fine. However if it starts before the access window the C code gets grumpy.
  # Here I fix negative startcol and startrow values (clipping the matrix and shifting the start values)
  rowshift <- colshift <- 0  # number of rows to shift.
  if(startrow < 1) {
    rowshift <- 1 - startrow
    startrow <- 1
  }
  if(startcol < 1){
    colshift <- 1 - startcol
    startcol <- 1
  }
 
  if(rowshift != 0 | colshift !=0){
    if(rowshift >= nrow(x) | colshift >= ncol(x)){
      warning("The grid doesn't overlap the access window.  Nothing was written.")
      return(invisible())
    }
    x <- x[(1+rowshift):nrow(x), (1+colshift):ncol(x)]
  }
  
  # Note writeblock will return an error if the grid doesn't exist here we assume it exists 
  # but capture errors and create the grid and try again if it was missing.
  #
  
  if(any(is.na(c(startrow, startcol, nrow(x), ncol(x), na.value)))) stop("NA inputs to .writeblock")
  for(i in 1:2){ # at most we try twice, most of the time we will break after one time.  However, 
  	# if the C function error indicates the grid didn't exist then we create the 
  	#  grid, don't break the loop,  and the block write is done over.
  	
    # C Function decleration:
    # void R_writeblock_dbl(int *ret_v, char **grid_name, int *y_start, int *x_start, 
    #    int *y_count, int *x_count, double *missing_value, double *mtx_block, int *transparency) ;
    done <- FALSE
    .status$gsdowntime <- 0  # The amount of time the gridserver has been down
    while(!done){  
      st <- system.time({
        if(as.integer){
          r <- .C("R_writeblock_int", integer(1), as.character(path), as.integer(startrow), 
                  as.integer(startcol), as.integer(nrow(x)), as.integer(ncol(x)), 
                  as.integer(na.value), as.integer(t(x)),as.integer(tc), PACKAGE="caps_lib") 
        } else {  # real (double)
          r <- .C("R_writeblock_dbl", integer(1), as.character(path), as.integer(startrow), 
                  as.integer(startcol), as.integer(nrow(x)), as.integer(ncol(x)), 
                  as.double(na.value), as.double(t(x)),as.integer(tc), PACKAGE="caps_lib") 
        }
      })
    .status$gridwait <- .status$gridwait + as.numeric(st[3])
      done <- .donetrying(code=r[[1]], st, reset=TRUE)    
    }
  
    if(lookup.error(r[[1]]) %in% c("GRID_SERVER_ACCESS_ERR_GRIDCANNOTFIND")){ # 
      gridcreate(path, as.integer=as.integer)
    } else { # no error or an error other than the one that indicates the grid doesn't exist
      break  # than we are done 
    }
  } # END LOOP
  if(r[[1]] < 0){
    a <- checkgrid(path, pathonly = TRUE)
    if(a$problem){
      a$problem <- NULL
      stop(".writeblock error with ", path," : ", paste(names(a)[unlist(a)], collapse = ", "))
    }
    parseerror(r[[1]], paste("writeblock", ifelse(as.integer, "(integer)", "(real)")) )
  }  
  
  if(path %in% ls(.status$gridinfo))
    rm(list=path, envir = .status$gridinfo)
  invisible()
}

