kernelsmooth.character <- function (x, 
                                    kernelfun,  
                                    max.r, 
                                    kernel.dim = 21, 
                                    outgrid, 
                                    overwrite = FALSE, 
                                    tilesize = 2000, 
                                    max.p.na = 1, 
                                    sample.points, 
                                    weights, 
                                    no.match = 0, 
                                    na.value = NA,
                                    ...) {
  # Function to peform kernel based  smoothing to grids on disk using any 
  #  type of kernel.
  # Arguments :
  #  x : the path to the grid that will be smoothed
  #  kernelfun :  a function to generate a kernel. It must have arguments
  #     mar.r and cellsize.  additional arguments (...) will also be 
  #     passed onto kernelfun.
  #     
  
  if (!getwindow()$set) 
    stop("gridinit must be called prior to this function.")
  xinfo <- griddescribe(x)
  if (xinfo$type == "missing") 
    stop("Grid", x, "not found.")
  if (!coincide(getwindow(), xinfo) )
    stop("Window extent does not match grid. Call setwindow first.")
  
  if (missing(outgrid)) {
    no.out <- TRUE
  }
  else {
    no.out <- FALSE
    if (griddescribe(outgrid)$type != "missing") {
      if (overwrite) 
        gridkill(outgrid)
      else stop("grid \"", outgrid, "\" already exists use overwrite=TRUE or a different output grid.")
    }
    gridcreate(outgrid, as.integer = FALSE)
  }
  if (missing(sample.points) & no.out) 
    stop("There's no sense in running if you don't specify either an outgrid or sample.points")
  if (!missing(sample.points)) {
    if (!all(c("x", "y") %in% names(sample.points))) 
      stop("sample.points must have columns named \"x\" and \"y\"")
    sampled.values <- numeric(nrow(sample.points))
    sampled.values[] <- NA
    sample = TRUE
  }
  else {
    sample = FALSE
  }
  current.window.dim <- max.r * 2/xinfo$cellsize
  factor <- max(1, ceiling(current.window.dim/kernel.dim))
  tilesize <- ceiling(tilesize/factor) * factor
  cat("Upscaling factor:", factor, "(cells)\nRealized tilesize:", 
      tilesize, "(cells)\n")
  buffer = ceiling(max.r/xinfo$cellsize/factor) * factor + 
    factor * 2
  
    cat("Buffer:", 0, "\n  - Using Method 3\n")
    tileinit(tilesize, buffer = 0) 
    cellsize <- xinfo$cellsize * factor 
    ntiles <- tilescheme()$totaltiles
    nr <- ceiling(xinfo$nrow/factor) + 2
    nc <- ceiling(xinfo$ncol/factor) + 2
    oyul <- xinfo$yll + xinfo$cellsize * xinfo$nrow
    yul <- oyul + cellsize
    yll <- yul - nr * cellsize
    xll <- xinfo$xll - cellsize
    ug <- list(m = matrix(NA, nrow = nr, ncol = nc), nrow = nr, 
               ncol = nc, xll = xll, yll = yll, cellsize = cellsize)
    class(ug) <- c("grid", class(ug))
    
  
  for (i in 1:ntiles) {
      settile(i)
      cat("Reading tile", i, "of", ntiles, "\n")
      g <- readtile(x)
      if (!missing(weights)) {
        if (all(c("value", "weight") %in% colnames(weights))) {
          g <- swap(g, weights$value, weights$weight, 
                    no.match = no.match, na.value = NA)
        }
        else if (ncol(weights) == 2) {
          g <- swap(g, weights[1, ], weights[, 2], no.match = no.match, na.value = na.value)
        }
        else {
          stop("If weights are supplied and the dataframe has more than two columns it must have columns labled \"value\" and \"weight\".")
        }
      }
      g <- upscale(g, factor = factor, max.p.na = max.p.na)
      startcol <- 2 + (tilescheme()$j - 1) * tilescheme()$colsize/factor
      endcol <- startcol + g$ncol - 1
      startrow <- 2 + (tilescheme()$i - 1) * tilescheme()$rowsize/factor
      endrow <- startrow + g$nrow - 1
      ug$m[startrow:endrow, startcol:endcol] <- g$m
    }
  
  
    cat("Smoothing entire upscaled grid\n")
    
    kern <- kernelfun(max.r=max.r, cellsize=cellsize, ...)
    ug<-kernelsmooth(ug, kern)
    
    for (i in 1:ntiles) {
      settile(i)
      startcol <- 1 + (tilescheme()$j - 1) * tilescheme()$colsize/factor
      endcol <- startcol + ceiling(tiledetails()$ncol/factor) + 
        1
      startrow <- 1 + (tilescheme()$i - 1) * tilescheme()$rowsize/factor
      endrow <- startrow + ceiling(tiledetails()$nrow/factor) + 
        1
      cellsize <- ug$cellsize
      ncol = endcol - startcol + 1
      nrow = endrow - startrow + 1
      xll = ug$xll + (startcol - 1) * cellsize
      yll = ug$yll + (ug$nrow - endrow) * cellsize
      sg <- list(m = ug$m[startrow:endrow, startcol:endcol], 
                 nrow = nrow, ncol = ncol, xll = xll, yll = yll, 
                 cellsize = cellsize)
      class(sg) <- c("grid", class(sg))
      cat("Downscaling and writing tile", i, "\n")
      sg <- downscale(sg, match.extent = TRUE, factor = factor)
      sg <- matchextent(sg, tiledetails())
      if (sample) {
        tc <- tiledetails()
        xmin <- tc$xll
        ymin <- tc$yll
        xmax <- tc$xll + tc$ncol * tc$cellsize
        ymax <- tc$yll + tc$nrow * tc$cellsize
        sv <- sample.points$x > xmin & sample.points$x <= 
          xmax & sample.points$y > ymin & sample.points$y <= 
          ymax
        rowcol <- matrix(NA, nrow = sum(sv), ncol = 2)
        rowcol[, 1] <- y2r(sample.points$y[sv], sg)
        rowcol[, 2] <- x2c(sample.points$x[sv], sg)
        sampled.values[sv] <- sg$m[rowcol]
      }
      if (!no.out) 
        writetile(sg, outgrid)
    }
if (sample) 
  return(sampled.values)
}
