mgridpredict <- function(fit, grids, na.rm=TRUE, list=NULL,  force=NULL, ...){
  # This function makes predictions on gridded data in memory
  #   see gridpredict for making predictions on grids on disk.
  #
  # Arguments
  #  fit - a fitted model on which the predict function can be called
  #z 	grids - a list of grid objects
  #	vars - a vector of variable names one for each grid in grids
  #	gridout - a path to an output grid
  #	tilenumber - the tilenumber to process
  #  na.rm - if TRUE all cells that have NA in any of the input grids won't be passed to fit (and will receive NA) otherwise whatever data is available in the grid for those cells will be passeed to fit.
  #  list - If predictions are desired from more than one fit object than they can be passed as a list via this argument
  #		In which case outgrid should be a vector of output grids one for each fit in list.
  #	force - a named list of variable names and values they should take on. This is to force some covariates to take on a single value.  We need this funcitonality for one narrow use but don't anticipate it to be widly used. 	NULL indicates that no variables will be forced.
  #	... - other arguments to be passed to the predict when called on fit
  if(!is.list(grids) || is.null(names(grids)) || any(is.na(names(grids)))) stop("grids must be a named list of grids.")
  ng <- length(grids)   
  vars <- names(grids)
  if(ng > 1) for(i in 2:(ng)) for(j in c("nrow", "ncol", "cellsize", "xll", "yll"))    
    if(grids[[1]][[j]] != grids[[i]][[j]]) stop("Grids don't allign: ", j, " in ", 
                                                 names(grids[1]), " doesn't match ", names(grids[i]), sep="")
  # make empty matrix to store the data
  d <- matrix(NA, ncol=(ng+length(force)), nrow=(grids[[1]]$nrow * grids[[1]]$ncol)) 
  d <- as.data.frame(d)
  
  # Setup so that list is the list of fits to fit
  if(is.null(list)){
    list <- list(fit=fit)
    list.used <- FALSE
  } else {
    list.used <- TRUE
  }

  # Make na.rm a vector of the same length as list
  if(!is.logical(na.rm)) stop("na.rm must be TRUE or FALSE")
  if(length(na.rm) == 1) na.rm = rep(na.rm, length(list))   
  if(length(na.rm) != length(list)) stop("na.rm must be either a scalar (to be used on all fits) or a vector of the same length as list.")
       
  # assemble matrix of prediction data from grids
  for(i in 1:ng){
    d[,i] <- as.vector(grids[[i]]$m)
    colnames(d)[i] <- names(grids)[i]
  }

  # add forced columns
  if(!is.null(force)) for(i in 1:length(force)){
    d[, (ng+i)] <- as.numeric(force[[i]])
    colnames(d)[(ng+i)] <- names(force[i])
  }
  cat("Done assembling dataframe\n")

  
  result <- list()
  na.sv <- apply(d, 1, function(x) any(is.na(x)))  # selection vector to select na-containing rows    
      
  # loop through models (only one if fit argument was used) make predictions and write out results
  for(i in 1:length(list)){
    fit <- list[[i]]
    cat("Making prediction\n")
    if(na.rm[i]){    
      pred <- vector(mode="numeric", length=nrow(d)) # preallocate prediction vector
      pred[]<- NA
      pred[!na.sv] <- predict(fit, newdata=d[!na.sv, , drop=FALSE] , ...)  
    } else {
      pred <- predict(fit, newdata=d, ...)
    }
    result[[i]] <- grids[[1]] # copy input grid
    result[[i]]$m <- matrix(pred, nrow=grids[[1]]$nrow, ncol=grids[[1]]$ncol) # overwrite data with result
    names(result)[i] <- names(list)[i] # copy over name from fit list
  }
  
  # if the fit argument was used return a single grid (not in a list) otherwise return a list
  if(list.used) return(result) else return(result[[1]])
  
}
