
# Example usage
if(FALSE){  # Skip example usage when sourcing the file (sourcing just defines the function)
 
  ### Setup a temporary directory with a sample grid in it 
  # Get path to example dem grid (included with package)
  require(gridio)
  require(anthill)
  datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
  datapath <- shortPathName(datapath)
  datapath <- paste(datapath, "/.", sep="")
  
  # Make a temporary directory to write new grids to
  dir <- tempdir() # get temporary directory
  dir.create(dir)
  
  # Copy example data into temporary directory
  file.copy(datapath, dir, recursive=TRUE)
  # system(paste("open", dir)) # if you want to look at contents of temporary directory
  ###  Done setting up directory
  
  cleanup() # disconnect from any prior gridservers 
  gridinit(dir) # initialize local gridserver with base path in temporary dir
  setwindow("dem") # setwindow to dem file
  
  ### Make up some sample data
  g <- readgrid("dem")
  gradient <- seq(20, 100, length.out=g$nrow)
  g$m[,] <- gradient 
  g$m <- g$m + rnorm(g$nrow*g$ncol)
  writegrid(g, "gradient")
  elev <- readgrid("dem")
  grad <- g
  d <- data.frame(elev=as.numeric(elev$m), grad=as.numeric(grad$m)) 
  d$y <- d$elev + d$grad + rnorm(nrow(d), 0, 10)
  obs <- g
  obs$m <- matrix(d$y, nrow=obs$nrow, ncol=obs$ncol)
  plot(obs)
  writegrid(obs, "obs")
  
  # Make fit object
  f <- lm(y~elev+grad, data=d)
  dir <- gsub("\\\\", "/", dir)
  
  path <- "X:/anthill/control"  # ethan's computer
  path <- "x:/anthill"  # cluster

  #### The real example starts here
    
  # Make sure lockserver and gridserver are running prior to executing the rest  
  config(path)

  tilepredict.args <- list(fit=f, grids=paste(dir, "/", c("dem", "gradient"), sep=""), 
                           vars=c("elev", "grad"),outgrid=paste(dir, "/pred",sep=""))
  
  pgridpredict(tilepredict.args=tilepredict.args, gridserver="EthanXP", 
               port=3333, path=path, name="pgridpredicttest", 
               refgrid=tilepredict.args$grids[1], tilesize=100, maxthreads=3)  # Do not use 100 as your tilesize!
  
}





pgridpredict <- function(tilepredict.args, # named list of arguments to tilepredict (tilenumber can be ommitted)
                         tilesize,   # the tilesize to use 
                         tiles,           # An integer vector indicating a subset of tiles to process (optional)
                         refgrid,         # The reference grid used to set up the gridserver (can be any grid of the right extent and cellsize)
                         gridserver,      # the name of the gridserver 
                         port,            # the port associated with the gridserver
                         name,            # the project name (NOT optional)
                         owner,           # The project owner (NOT optional)
                         refmosaic, # Path to reference mosaic
                         ...              # other arguments to launch.project such as:
                                          #   priority, and maxthreads
){
  # pgridpredict  
  # Ethan Plunkett July 30, 2013
  
  # This is a parallel version of grid predict.  It uses anthill to launch a project on the cluster
  #  with one task with subtasks for each tile in landscape.
  # It's recommended that maxthreads be set low initialy and then increased only until gridwait
  #  begins to increase. 
  
  # Validate input on tiles and tilesize
  if(!missing(tiles)) if(missing(tilesize)) stop("Tilesize must be specified if using the tiles argument.")
  if(missing(tilesize)) tilesize=2000
  if(!anthillrunning())
    stop("Anthill must be running for this function to work.")
  
  # Get task data
  path <- .settings$path
  td <- read.task.data()
  unlock()
  
  # Validate the project name (must be unique)
  new.name <- name
  count <- 2
  while(new.name %in% td$projects$project){
    new.name <- paste(name, count)
    count <- count + 1 
  }
  if(new.name != name){
    cat(name, " was already in use.  Using \"", new.name, "\" instead.", sep="")
    name <- new.name
  } 
  
  # Figure out the project directory and the project workspace path
  # Note: I'm using the same rlaunch subdirectory as simple.launch so
  #  that the files will be cleaned up automatically by purge project.
  project.dir <- paste(path, "/rlaunch/", name, sep = "")
  if (!file.exists(project.dir)) 
    dir.create(project.dir, recursive=TRUE)
  
  loc <- paste(project.dir, "/", name, "-", format(Sys.time(), 
                                                   "%Y-%m-%d"), ".Rdata", sep = "")
  i <- 2
  ol <- loc
  while (file.exists(loc)) {
    loc <- gsub(".Rdata$", paste("_", i, ".Rdata", sep = ""), 
                ol)
    i <- i + 1
  }
  
  # Check gridserver
  cleanup()
  gridinit(server=gridserver, port=port, log=FALSE)
  refdesc <- griddescribe(refgrid)
  if(refdesc$type == "missing")stop("The reference grid ('", refgrid , "'') doesn't exist.", sep="")
  w <- getwindow()
  if(!w$set){
    setwindow(refgrid)
  } else {
    if(!coincide(w, refdesc))
      stop("The gridserver: ", gridserver, "port:", port, "is already set to a refrence grid that:", 
           "doesn't match:", refgrid )  
  }
  
  call.mosaicinit <- FALSE
  if(!missing(refmosaic)){
    mosaicinit(reference=refmosaic, servers=readgridservers())
    call.mosaicinit <- TRUE
  
  }
  
  # Generate or check the tile list
  tileinit(tilesize, buffer=0)
  if(missing(tiles)){
    tiles <- 1:ntiles()
  } else {
    stopifnot(all(tiles %in% 1:ntiles()))
  }
  
  # Make function to do the actual predictions
  predtile <- function(tile){
    cleanup()
    gridinit(server=gridserver, port=port)
    if(call.mosaicinit) 
      mosaicinit(reference=refmosaic, servers=readgridservers())
    tileinit(tilesize, buffer=0)
    tilepredict.args$tilenumber <- tile
    do.call(tilepredict, args=tilepredict.args)
  }
  
  # build tasklist
  tasklist <- data.frame(command="predtile()", 
                         subtask=paste("range:", compact.range(tiles), sep=""), 
                         subtaskarg="tile", 
                         system="R", 
                         workspace=loc
  )
  
  # Make workspace 
  l <- ls(envir=.GlobalEnv)
  l <- unique(c("tilepredict.args", "tilesize", "gridserver", "local", "port", 
         "predtile", l))
  save(list=l,file=loc)
  
  launch.project(name=name, tasklist=tasklist, directory=project.dir, path=path, owner=owner, ...)
  
}
