
readblockc <- function(path, startrow, startcol, nrow, ncol, buffer=0, extent, as.matrix=FALSE, 
                       na.value=-9999, as.integer, cacheok, desc){
  # Note readblock now spans three functions
  # readblock - handles mosaics calling readblockc repeatedly.
  # readblockc - handles caching calling .readblock once on either a cached copy or the source
  # .readblock - interfaces directly with the dll. 
  #
  #
  #  Changes : May 2016 Reworked to allow hashing of the gridpath to map to 
  #   1 of a large number of index files. This means for reads the whole index
  #   doesn't need to be read and written.  
  #     https://docs.google.com/document/d/1-CpmA_D8jBADkn8SVMcqeySEvLPtXoi4AAuNWkYsO_0/edit?usp=sharing
  #  Has psuedo code and locking logic for this update.
  #
  
  
  # Set conversion factor.  Note if changin here change in fileinfo as well
  # mb.per.gb <- 1024 # base 2
  mb.per.gb <- .status$mb.per.gb # base 10 
  
  mingridcapacity = 10 # the minimum space available in the cache expressed as a multiple of the 
  # current grid size.  Thus with 10 we are only going to cache if the 
  # total cache space is at least 10 times the size of the current grid
  
  cachetimeout<- anthill:::.settings$cachetimeout  # Cachetimeout is set in anthill config file 
                                                   # Brad made that decision.  It's OK, I guess,
                                                    # b/c caching is already dependent on anthill
                                                  # value is in minutes
  
  verbose <- .status$verbosity >= 3
  if(.status$verbosity >= 2) cat("reablockc\n")
  if(.status$verbosity >= 4) cat("Arguments:\n\t", 
                                 "path =", path, "\n\t", 
                                 "startrow = ", ifelse(missing(startrow), "[missing]",  startrow), "\n\t", 
                                 "startcol = ", ifelse(missing(startcol), "[missing]",  startcol), "\n\t", 
                                 "nrow = ", ifelse(missing(nrow), "[missing]", nrow), "\n\t", 
                                 "ncol = ", ifelse(missing(ncol), "[missing]", ncol), "\n\t",
                                 "buffer = ", buffer, "\n\t",
                                 "extent = ", ifelse(missing(extent), "[missing]", "[not missing]"),   "\n\t", 
                                 "as.matrix = ", as.matrix, "\n\t", 
                                 "na.value = ", na.value, "\n\t", 
                                 "as.integer = ", ifelse(missing(as.integer), "[missing]", as.integer), "\n\t", 
                                 "cacheok = ", ifelse(missing(cacheok), "[missing]", cacheok),  "\n\t", 
                                 "desc = ", ifelse(missing(desc), "[missing]", "[not missing]"), 
                                 "\n") 
  path <- gsub("/[[:blank:]]*$", "", path)
  
  # Check if caching is on
  ci <- getcacheconfig()
  if(!ci$on) cacheok <- FALSE  
  
  if(cacheok){ # Main cacheok code block (checks for grid in cache and attempt to copy it in if missing)
    library(anthill)
    
    # Get information on source grid and throw error if it doesn't exist
    si <- fileinfo(path) 
    if(is.na(si$size)){
      stop("There is no grid:", path)
    }      
    source.drive <- gsub("/.*", "",  path) 
    cacheinplace <- grepl(paste("^" , source.drive, sep=""), ci$cachepath, ignore.case=TRUE)
  }
  
  if(cacheok && !cacheinplace){
    # Setup for locking 
    main.lock.path <- paste0(short.name(), " cache") 
    index.path <- getcachepath(path)
    index.lock.path <- paste0(short.name(), " ", index.path)
    msg <- paste(short.name(), "-R", sep="")
    
    # Get index and lock
    start.cache.timer()
    get.lock(index.lock.path, message=msg, slash=FALSE)
    on.exit({ # in case there are errors
      return.lock(index.lock.path, slash=FALSE)
      stop.cache.timer()
    })
    d <- getcachedir(path, locking=FALSE)
    
    ri  <- lookupgrid(path, d)
    iscached <- !is.na(ri)
    if(iscached){  
      if(verbose) cat("\tgrid is already in the cache\n")
      # If someone else is copying our grid into the cache then wait for them to finish
      first <- TRUE  # first time through the loop (for print statement only)
      while(d$status[ri] %in% "copying"){
        # Check that it hasn't been copying for more than 10 minutes 
        tdiff <- difftime(Sys.time(),  d$cachedate[ri])
        units(tdiff) <- "mins"
        if(as.numeric(tdiff) > cachetimeout){
          cat("Cached version of '", path, "' has been 'copying' for more than ", cachetimeout, " minutes on '",
               short.name(), "'. Starting new copy.\n")
          iscached <- FALSE
          break
        }
        if(verbose){
          if(first == TRUE){
            cat("\tWaiting for someone else to copy the grid from the cache.\n")
            first <- FALSE
          } else {
            cat(".") 
          }
        }
        msg <- paste(short.name(), "-R waiting for copying to complete", sep="")
        return.lock(index.lock.path, message=msg, slash=FALSE)
        stop.cache.timer()
        on.exit()
        Sys.sleep(120)
        
        start.cache.timer()
        get.lock(index.lock.path, message=msg, slash=FALSE)
        on.exit({
          return.lock(index.lock.path, slash=FALSE)
          stop.cache.timer()
        }) # in case there are errors
        d <- getcachedir(path, locking=FALSE)
        ri  <- lookupgrid(path, d)
        if(is.na(ri)) {
          iscached <- FALSE
          break
        }
      }  # Done waiting for copying to finish
      
      if(iscached  && !(d$status[ri] %in% "ready")) 
        stop("Grid is no longer copying but isn't 'ready'.  Something is wrong.")
      
      # Check for expired grid
      if(iscached && d$sourcedate[ri] != si$date){  
        if(verbose) cat("Cached grid is expired.\n")
        iscached <- FALSE
        d$status[ri] <- "expired"  # this tells friskcache to delete the grid from the index
        d$source[ri] <- "expired"  # this means that when we look up a grid by path expired grids won't match
      } # end check for expired grids
      
    } # End Grid was already in cache
    
    
    # if it's not cached then add the grid to the cache
    if(!iscached ){    
      if(verbose)  cat("Adding grid to cache\n")
      if(ismultitiled(path)){  # Multititled grids can't be read safely and thus shouldn't be cached 
        # putcachedir(d = d, path) # shouldn't have changed so we can leave as is.
        stop("Grid (", path, ") is multitiled and can't be read.  See makemosaic for a work around.")
      }
      
      # Add row to index indicating that we are going to copy this grid.
      name <- gsub("^.*/", "", path)
      cachedate <- Sys.time()
      newrow <- data.frame(source=gsub("/", "\\\\", path), 
                           cache= "",
                           status="copying", 
                           cachedate=cachedate, 
                           sourcedate=si$date, 
                           reads=NA, 
                           size=si$size, 
                           stringsAsFactors=FALSE)
      d <- rbind(d, newrow)
      putcachedir(d = d, path)          # need to release index lock prior to getting 
      return.lock(index.lock.path, slash = FALSE)      #    cacheinfo lock to avoid deadlock
      stop.cache.timer()
      on.exit(NULL)
      
      start.cache.timer()
      get.lock(main.lock.path, slash = FALSE)
      on.exit({
        return.lock(main.lock.path, slash = FALSE)
        stop.cache.timer()
      })
      
      a <- readLines(paste0(ci$cachepath, "/index/cacheinfo.txt") )
      last.id <- as.numeric(a[[1]])
      cachesize <- as.numeric(a[[2]])  # cache size in MB
      
      free <- fileinfo(paste0(ci$cachepath, "/index/cacheinfo.txt"))$free
      tot.available <- min(free + cachesize - ci$minfree*mb.per.gb, ci$maxsize*mb.per.gb)
      copying <- TRUE  
      if(tot.available < ( mingridcapacity * si$size) ){
        copying <- FALSE
        cacheok <- FALSE
      } else {
        available <- min(free - ci$minfree*mb.per.gb , ci$maxsize* mb.per.gb - cachesize)  # unused capacity
        if(available < si$size){  # If not enough free space in cache
          delete.size <- max(si$size, ci$deletesize * mb.per.gb)    
          succeeded <- friskcache( freeup = delete.size, deletecopies = FALSE, getlock = FALSE)
          if(!succeeded){
            copying <- FALSE
            cacheok <- FALSE
          }          
          # Reread the cachesize b/c it was changed by friskgrids
          a <- readLines(paste0(ci$cachepath, "/index/cacheinfo.txt") )
          last.id <- as.numeric(a[[1]]) # last id used in index
          cachesize <- as.numeric(a[[2]]) # cache size in MB
        } # end IF insuficient space
      } # End Checks for space
      
      if(copying){ # if copying figure out cache.path and update the cachesize and last.id on disk
        new.id <- last.id + 1
        new.size <- (cachesize + si$size) 
        writeLines(as.character(c(new.id, new.size)), con = paste0(ci$cachepath, "/index/cacheinfo.txt") )
      }
      
      return.lock(main.lock.path, slash = FALSE)  # release lock on cacheinfo  
      stop.cache.timer()
      on.exit(NULL)  # No locks held
      
      if(copying){  # Actually copy the grid!!!
        if(verbose) cat("Copying grid to cache. - ")
        cache.dir <-  paste(gsub("(/|\\\\)$", "", ci$cachepath), "/", new.id, sep="")
        if(file.exists(cache.dir)) unlink(cache.dir, recursive=TRUE)
        dir.create(cache.dir)
        cache.path <- paste(cache.dir, "/", name, sep="")
        setcache(TRUE)
        activateconnection()
        gridcopy(path, cache.path)
        if(verbose) cat("Done copying grid to cache.\n")
      }
      
      # Now update the index file to record either the successful copy or the failed copy
      start.cache.timer()
      get.lock(index.lock.path, message=msg, slash=FALSE)
      on.exit({ # in case there are errors
        return.lock(index.lock.path, slash=FALSE)
        stop.cache.timer()
      })
      d <- getcachedir(path, locking=FALSE)
      ri  <- lookupgrid(path, d)
      if(copying) {
        d$cache[ri] = gsub("/", "\\\\", cache.path) 
        d$status[ri] <- "ready"
      } else { # Did not Copy
        if(verbose) cat("Copying skipped b/c of insufficient space. \n")
        d <- d[-ri, ]  # drop copying line from index b/c we didn't actually copy anything there          
        putcachedir(d, path) 
        return.lock(index.lock.path, message=msg, slash=FALSE) 
        stop.cache.timer()
        on.exit()
      }
      
    } # End not cached (done adding grid to index)
  } # End cacheok 
  
  if(cacheok){  # If still reading from cache
    # Update read time(s)
    if(!cacheinplace){
      reads <- d$reads[ri] 
      now <- format(Sys.time(), format=.status$timeformat)
      if(is.na(reads)){
        reads <- now
      } else {
        reads <- strsplit(reads, "/")[[1]]
        n <- length(reads)
        if(n > 4) reads <- reads[n-3:n]  # drop all but last four timestamps
        reads <- c(reads, now)  # add this one
        reads <- paste(reads, collapse="/")
      }
      d$reads[ri] <- reads
      putcachedir(d, path) 
      return.lock(index.lock.path, message=msg, slash=FALSE) 
      stop.cache.timer()
      on.exit()
    }
    
    
    readfrom <- ifelse(cacheinplace, path, d$cache[ri])    
    setcache(TRUE)
    activateconnection()
    if(verbose) cat("Reading grid from cache\n")
    return(.readblock(path = readfrom, startrow=startrow, startcol=startcol, nrow=nrow, ncol=ncol, 
                      buffer=buffer, extent=extent, as.matrix=as.matrix, na.value=na.value, 
                      as.integer=as.integer, desc=desc))
    
  } else {  # Otherwise read grid directly
    
    if(verbose) cat("Reading uncached grid.\n")
    setcache(FALSE)
    activateconnection()
    if(ismultitiled(path)) 
      stop("Grid (", path, ") is multitiled and can't be read.  See makemosaic for a work around.")
    
    return(.readblock(path=path, startrow=startrow, startcol=startcol, nrow=nrow, ncol=ncol, 
                      buffer=buffer, extent=extent, as.matrix=as.matrix, na.value=na.value, 
                      as.integer=as.integer, desc=desc))
  }
  
  
} # End function Definition



