tilepredict <- function(fit, grids, vars, outgrid, tilenumber, na.rm=TRUE, list=NULL, 
                        force=NULL, na.value = -9999, as.integer=FALSE, cacheok=TRUE, as.mosaic, group = NULL, ...){
  # Arguments
  #	fit - a fitted model on which the predict function can be called
  # 	grids - a vector of gridpaths 
  #	vars - a vector of variable names one for each grid in grids
  #	gridout - a path to an output grid
  #	tilenumber - the tilenumber to process
  #  na.rm - if TRUE all cells that have NA in any of the input grids won't be passed to fit (and will receive NA) otherwise whatever data is available in the grid for those cells will be passeed to fit.
  #  list - If predictions are desired from more than one fit object than they can be passed as a list via this argument
  #		In which case outgrid should be a vector of output grids one for each fit in list.
  #	force - a named list of variable names and values they should take on. This is to force some covariates to take on a single value.  We need this funcitonality for one narrow use but don't anticipate it to be widly used. 	NULL indicates that no variables will be forced.
  # na.value - this value is used to transmit NA to the underlying C code. Set it to a value that doesn't occur in your dataset (default is -9999)
  # as.integer - Indicates whether each new grid will be floating point or integer.  Can be a vector of the same length as list
  # group - (optional)  path to an integer grid which specifies which models should be applied to which part of the landscape
  #     if specified then the list argument should be used, the list items should have integer names matching classes in the grouping grid, and only one outgrid should be specified.
  
  #	... - other arguments to be passed to the predict when called on fit
  
  if(!getwindow()$set) stop("gridinit must be called prior to this function.")
  if(is.na(tilescheme()$totaltiles)) stop("tileinit must be called prior to this function")
  if(missing(as.mosaic)) as.mosaic <- .status$as.mosaic
  
  ng <- length(grids) 
  if(length(vars) != ng ) stop("grids and vars should be of the same length.")
  
  settile(tilenumber)
  td <- tiledetails()
  
  d <- matrix(NA, ncol=(length(grids)+length(force)), nrow=(td$nrow*td$ncol)) # make empty matrix to store the data
  d <- as.data.frame(d)
  
  # Setup so that list is the list of fits to fit
  if(is.null(list)){
    list <- list(fit=fit)
  } else {
    if(!is.null(group)){
      if(length(group)!= 1  & !is.character(group)) 
        stop("Group should be the path to a single grid")
      if(is.null(list))
        list <- list(fit=fit)
      if(length(outgrid)!= 1) stop("If you use a group grid their should only be one output grid.")
    } else {
      # (no group grid)
      if(length(list)!= length(outgrid)) stop("If the list argument is used there must be the same number of elements in it and outgrid")  
    }
  }
  
  
  if(length(as.integer)==1) as.integer <- rep(as.integer, length(list))
  if(length(as.integer)!= length(list)) 
    stop("If as.integer is specified it must either be a single value to be used for all models or a vector of values of the same length as list")
  
  # assemble matrix of prediction data from grids
  for(i in 1:ng){
    cat("Reading tile ", tilenumber, " from grid \"", grids[i], "\"\n", sep="")
    g <- readtile(grids[i], na.value=na.value, cacheok=cacheok)
    if(na.rm && all(is.na(g$m))){
      cat("First grid is all NA in this tile.  Moving on.")
      return() 
    }
    d[,i] <- as.vector(g$m)
    colnames(d)[i] <- vars[i]
  }
  cat("Done reading grids\n")
  
  # add forced columns
  if(!is.null(force)) for(i in 1:length(force)){
    d[, (ng+i)] <- as.numeric(force[[i]])
    colnames(d)[(ng+i)] <- names(force[i])
  }
  
  # loop through models (only one if fit argument was used) make predictions and write out results
  if(is.null(group)){
    
    for(i in 1:length(list)){
      fit <- list[[i]]
      cat("Making prediction\n")
      if(na.rm){
        sv <- apply(d, 1, function(x) !any(is.na(x)))  # selection vector to select non-na-containing rows
        if(sum(sv) == 0){
          cat("All NA. Moving on.")
          return() # nothing to do
        }
        pred <- vector(mode="numeric", length=nrow(d)) # preallocate prediction vector
        pred[]<- NA
        pred[sv] <- predict(fit, newdata=d[sv, , drop=FALSE] , ...)  
      } else {
        pred <- predict(fit, newdata=d, ...)
      }
      cat("Writing results to \"", outgrid[i],"\"\n", sep="" )
      g$m <- matrix(pred, nrow=g$nrow, ncol=g$ncol)
      writetile(g, outgrid[i], na.value=na.value, as.integer=as.integer[i], as.mosaic=as.mosaic)
    }
  }  # end no group grid
  
  if(!is.null(group)){
    grp <- readtile(path = group, na.value=na.value, cacheok = cacheok)
    ug <- unique(grp$m) # unique groups
    ug <- ug[!is.na(ug)]
    fit.ids <- as.integer(names(list))
    pred <- vector(mode="numeric", length=nrow(d)) # preallocate prediction vector
    pred[]<- NA
    
    ug <- intersect(ug, fit.ids) # retain only the groups that occur in this tile
    if(length(ug) == 0)
      return()
    for(i in 1:length(ug)){ # loop through the unique groups that appear in this tile
      gr <- ug[i]
      if(i == 1 || i %% 10 == 0) cat("Making prediction", i, "of", length(ug), "group = ", gr, "\n")
      sv <- grp$m %in% gr
      fit <- list[[which(fit.ids == gr)]]
      pred[sv] <- predict(fit, newdata=d[sv, , drop=FALSE] , ...)    
    }  
    
    cat("Writing results to \"", outgrid,"\"\n", sep="" )
    g <- newtile()
    g$m <- matrix(pred, nrow=g$nrow, ncol=g$ncol)
    writetile(g, outgrid, na.value=na.value, as.integer=as.integer[1], as.mosaic=as.mosaic)
  }
  
}

