\name{gaussianinterpolate}
\alias{gaussianinterpolate}
\alias{gaussian.interpolate}
\title{
function to convert irregular points into a grid by performing a Gaussian weighted mean of the points near each cell.
}
\description{
This function calculates a Gaussian weighted mean of the point data in x at each cell in an output grid.
Cells that aren't within max.r of any points will be assigned a value of NA. If path is specified it will
output to disk otherwise it will return a grid object.  
}
\usage{
gaussianinterpolate(x, sd, data.col, refgrid, max.r = 3 * sd, buffer, 
                    cellsize, path, overwrite = FALSE, ...)
gaussian.interpolate(...) # depreciated use gaussianinterpolate instead

}
\arguments{
  \item{x}{
a \code{\link{SpatialPointsDataFrame}} which can be produced by calling \code{\link{coordinates}} on
a dataframe.
}
  \item{sd}{
The standard deviation of the Gaussian weighting function.
}
  \item{data.col}{
The column in \code{x} which contains the values associated with each point.
}
  \item{refgrid}{
This is used to define the extent, cellsize, and cell alignment of the output grid.  It can be any of the following (1) a grid object,  (2) the path to a grid on disk, (3) a list containing the items : "xll", "yll", "cellsize", "nrow", "ncol" such at the output of \code{\link{griddescribe}} and \code{\link{getwindow}}. 
}
  \item{max.r}{
Points beyond this distance will be assigned no weight in the mean.
}
  \item{buffer, cellsize}{  If \code{refgrid} is not used than the output grid will have this \code{cellsize} and an extent that extends beyond the extent of the points by a distance of \code{buffer} in all directions.

}
  \item{path}{ (optional) if the path to a grid is specified the function will write the results to that grid.  If it is not specified than the function will return the results.
}
  \item{overwrite}{ Only relevant if \code{path} is specified. If 
  code{TRUE} the function will overwrite a pre-existing grid. Otherwise it will produce an error if the grid already exists.
}
  \item{\dots}{
  Additional arguments passed on to \code{writegrid} such as \code{na.value}, \code{add}, and \code{transparent}
}
}
\details{
This function will fail if the pixel dimensions of the grid are too large to fit in memory.  A version could be written that processed tile by tile.
}
\value{
If \code{path} is not specified it returns a grid object where each cell contains a Gaussian weighted mean of the nearby points. If path is specified it writes that grid to disk and returns nothing.
}
\author{
Ethan Plunkett
}

\seealso{
See \code{\link{gaussiansmooth}} and \code{\link{kernsource}} for Gaussian smoothing a grid object 
and \code{\link{kernelsmooth}} for kernel based smoothing of grids using arbitrary kernels.
}
\examples{

  # Generate test data
  library(gridio)
  set.seed(1)
  n <- 100
  x <- runif(n, 0, 1000)
  y <- runif(n, 0, 1000)
  z <- rnorm(n, mean=10, sd=10) + y * .02  # Slight north south gradient with a lot of noise
  a <- data.frame(x,y, z)
  coordinates(a) <- c("x", "y")  # convert to SpatialPointsDataFrame
  
  # gaussianinterpolate
  g <- gaussianinterpolate(x=a, sd=50,max.r=300, data.col="z", buffer=50, cellsize=100)
  
  # Plot results
  plot(g)
  cols <- rgb(.5, 0, 0, (z - min(z) + .1 * (max(z)- min(z)))/(1.1 * (max(z)-min(z))) ) 
  cex <- (z - min(z) + .5 * (max(z)- min(z)))/(1.5 * (max(z)-min(z))) *2
  points(x, y, col=cols, pch=16, cex=cex)  
  
}
\keyword{ cluster }

