\name{gaussiansmooth}
\alias{gaussiansmooth.character}
\alias{gaussiansmooth}
\alias{gaussiansmooth.grid}
\alias{gaussian.smooth}
\title{
functions to apply a gaussian smoothing kernel to every cell in a grid. 
}
\description{
These functions return a grid of the same dimension as the input in which each cell has a gaussian weighted mean value of cells around it. To speed up processing with large bandwidths (SD) it upscales the data prior to applying a Gaussian kernel.  The amount of upscaling is determined by the kernel.dim argument which sets the dimension (in cells) of the kernel at the final resolution. The function can use a weights table to run directly on a landcover grid substituting the cover values with weights as defined in the weights table and with the no.match and na.value arguments.  Edge correction is used; NA cells and cells outside of the region are not included in the calculation.  
}
\usage{
gaussiansmooth(x, sd, max.r, kernel.dim, ...)
gaussian.smooth(...) # depreciated - now a wrapper to gaussiansmooth which should be used instead
\method{gaussiansmooth}{character} (x, sd, max.r = 3 * sd, kernel.dim = 21, outgrid, 
        overwrite = FALSE, tilesize, max.p.na=1, sample.points,weights, method, 
        no.match=0, tiles, cacheok=TRUE, use.old=FALSE, na.value = NA, ...)
\method{gaussiansmooth}{grid}(x, sd, max.r=3*sd, kernel.dim=9, max.p.na=1, 
        sample.points, weights, no.match=0, use.old = FALSE, na.value = NA, ...)
}

\arguments{
  \item{x}{
	\code{x} should either be a grid object in memory or a character representation of the path to a grid on disk.
}
  \item{sd}{
	The standard deviation of the kernel used to do the smoothing (in map units).
}
  \item{max.r}{
	The maximum radius that weights are calculated for (in map units).  It defaults to three times \code{sd}.
}
  \item{kernel.dim}{
      The dimension of the kernel in the upscaled grid. It must be an odd number.  This controls the amount of upscaling, which is adjusted such that a \code{max.r} radius kernel has this cell dimension.  Smaller values result in more upscaling and faster processing at the expense of precision. Use \code{Inf} for no upscaling.  Note the default value of this parameter was changed from 9 to 21 on 11/19/2015.
}
  \item{outgrid}{
	For the character method in which x is a path to a grid this argument specifies the path to the output grid.  If \code{sample.points} is specified this argument is optional.
}
  \item{overwrite}{
	For the character method this determines whether the output grid is overwritten.  If FALSE an error will be thrown if the grid already exists.
}
  \item{tilesize}{
	For the character method this determines the tilesize.  A bigger number will mean fewer, larger tiles.  If a tile size of 0 is specified the function will attempt to read and process the entire grid at once.  For small grids this may be more efficient.
}
  \item{max.p.na}{
	If the proportion of NA values (from the original resolution) in an upscaled
  cell exceeds \code{max.p.na} than it will receive an NA value during 
  upscaling; otherwise the upscaled cell is given the mean value of all 
  non-NA cells that it covers.
}		
 
 \item{sample.points}{
	(optional) a dataframe or matrix with columns "x" and "y" containing the map
  unit coordinates of points.  If specified the character method will return 
  the a vector of sample values (from the smoothed surface), and the grid 
  method will return both the samples and a grid (in a list).
}
\item{weights}{
	(optional) a dataframe or matrix with columns "value" and "weight".  If 
  \code{weights} is provided than all the cells in the grid will be replaced 
  with their associated weight; any cell whose value is not in the value column
  will be replaced with the values specified by \code{no.match} or \code{na.value}.  
  If \code{weights} does not contain columns
  named "value" and "weight" and contains only two columns it will be assumed
  that the first is "value" and the second is "weight".
}

\item{method}{
	(optional) either 1, 2, or 3.  This controls what method is used to generate
  the smoothed grid. All methods should produce the same result but depending 
  on other parameters they may be vastly different in terms of memory usage and
  speed.  Method 1 process the whole grid at once.  Method 2 reads and processes
  tiles sequentially. Method 3 reads, upscales, and stitches together tiles to 
  produce an upscaled version of the entire grid in memory; then smooths the
  entire grid; and finally downscales and writes one tile at a time.
}

\item{no.match}{ 
If weights are supplied this determines the value to use when cells in \code{x} 
 are not \code{NA} and do not match any of the values in \code{weights}. 
 }
 
 \item{na.value}{ 
If weights are supplied this determines the value to use when cells in \code{x} 
 are \code{NA}.  Note if the \code{weights} argument contains a value for  \code{NA} as well that will take precedence. 
 }

\item{tiles}{ Optionally specify a subset of tiles to process; this is useful 
if the landscape is irregular and thus large portions are NA.
If this argument is used then \code{tilesize} must also be provided.
 }
 
 \item{cacheok}{ If \code{TRUE}, the default, than cache the input grid if caching is on.
 }

  \item{use.old}{if \code{FALSE} then the newer \pkg{Rcpp} based implementation is used. 
  Otherwise the old, pure R, version is used.  This argument is here to 
  facilitate testing.}


 \item{\dots}{
	arguments to be passed to other methods.  In particular the character method calls the grid method so any argument of the grid method may be specified with the character method.
 }

}
\value{	
If \code{sample.points} are not provided and \code{x} is a grid then the function will return a grid, and if \code{x} is a path to a grid then the function will create a new grid on disk (\code{outgrid}) and return nothing. 
If \code{sample.points} are provided the grid method will return a two item list: \code{sample} containing a vector of sampled values, and \code{grid}; and the character method will return a vector of sample values.
}

\author{
Ethan Plunkett
}

\note{
Using bigger tile sizes is likely to increase performance.  It may also be faster to read the entire grid in at once (call with \code{method=1}).  On windows machines you can change your memory limit using \code{\link{memory.limit}}. 
It is left up to the user to make sure that the \code{weights} table is complete. Any values that occur in the grid but are not in the "value" column of \code{weights} will be replaced with \code{no.match} prior to smoothing unless they are \code{NA} in which case they will be replaced with \code{na.value}. 
Setting tilesize to 0 forces method 1 regardless of the state of the method argument.
Method 3 excels with large SD's.  With small SD's there may not be sufficient memory to store the entire upscaled grid and method 2 may be better. 
If the method argument is not supplied method 3 is used.  This will work well for most grids but may run out of memory if the grid is very large and the sd is small.  In those situations using method 2 will likely work well.    
}



\seealso{
\code{\link{kernelsmooth}}, \code{\link{makegaussiankernel}},  \code{\link{upscale}}, and  \code{\link{downscale}} are all called by this function.  
 \code{\link{kernelsmooth}} does the same thing but allows for non Gaussian kernels and is only efficient for smaller kernels because it doesn't do upscaling.
\code{\link{calckernel}} allows calculating a kernel weighted average for a single cell.
\code{\link{kernsource}} is similar but works by summing kernels centered on each cell, it may be faster if the non-zero values in the grid are sparse and the SD is small.
\code{swap} is used internally to substitute values when weights are used.
}
\examples{
#
}
\keyword{ spatial }
