\name{gridfun, tilefun, pgridfun}
\alias{tilefun}
\alias{gridfun}
\alias{pgridfun}
\title{
functions to apply functions models to gridded data 
}
\description{
These functions apply a function or list of functions to gridded data and write the result to grids.  \code{tilefun} works on a single tile. \code{gridfun} processes a whole grid in tiles. \code{pgridfun} uses \pkg{anthill} to process a grid in tiles in parallel.
}
\usage{
tilefun(FUN, grids, vars, outgrid, tilenumber, na.rm = TRUE, 
            na.value = -9999, as.integer=FALSE, cacheok=TRUE,
            as.mosaic, extra.args)
            
gridfun(FUN, grids, vars, outgrid, tilesize=2000, na.rm = TRUE, 
        na.value = -9999, overwrite = FALSE, as.integer=FALSE, 
        cacheok=TRUE, as.mosaic, extra.args, tiles)
            
pgridfun(tilefun.args, tilesize, tiles, refgrid, 
            gridserver, port, name, owner, refmosaic, ...) 
}

\arguments{
  \item{FUN}{
	A function or a list of functions to be applied to the grids.  Each function will be called with arguments set to vectors from the input grids with argument names matching \code{vars}.
}
  \item{grids}{
	A vector of grid paths specifying the data to use to make the predictions.
}
  \item{vars}{
	A character vector containing the variable name to be associated with each grid in \code{grids}.	
}
  \item{outgrid}{
	The path to an output grid. If \code{FUN} is a list of functions than the length of outgrid should match the length of \code{FUN}.
}

  \item{tilenumber}{
	The tile number to be processed.
}

  \item{tilesize}{
	The tilesize to use when processing grids. This is optional and defaults to 2000.  However, if the \code{tiles} argument is supplied than \code{tilesize} must also be specified.
}

  \item{na.rm}{
	If \code{TRUE} (the default) then any cell for which there is missing data in any of the input grids will not be passed to the function or functions but will instead receive NA.  If \code{FALSE} then data for all cells will be passed to the functions.
}

  \item{na.value}{
	(optional) This is the value used to represent \code{NA} values in the grid when passing data to the underlying C+ functions. It should be set to a value that does not occur in the data.
}

 \item{overwrite}{
  Specifies whether pre-existing grids should be overwritten.  If \code{FALSE} and \code{outgrid} already exists an error will be thrown.
}

 \item{as.integer}{
  Indicates format for the new grid(s) if \code{TRUE} they will be integer grids;  \code{FALSE} indicates that the new grid will contain floating point numbers.  If you are applying multiple functions than this should either be a single value which will be used for all output grids or a vector of \code{TRUE} and \code{FALSE} values that correspond to each output grid. The default value is \code{FALSE} which will make all output grids floating point. This is safe but possibly inefficient.
}

\item{cacheok}{if \code{TRUE} (the default) than the prediction grids are eligible for caching (if caching is configured). See \code{\link{cacheinit}}.
}

\item{as.mosaic}{if \code{TRUE} the output will be a mosaic. To work the user must call \code{\link{mosaicinit}} before calling \code{tilefun} or \code{gridfun}.  If calling \code{pgridfun} and you specify \code{as.mosaic=TRUE} in the \code{tilefun.args} list make sure to also use the \code{refmosaic} argument.  This argument is now optional and should default to the appropriate value most of the time; if mosaicing is setup it will write mosaics.}

  \item{tilefun.args}{
A named list of arguments and their values to be passed to \code{tilefun} the \code{tilenumber} argument
can be omitted.
}

  \item{tiles}{
An (optional) integer vector indicating a subset of tiles to process. 
}
\item{refgrid}{
The reference grid used to set up the gridserver (can be any grid of the right extent and cellsize).
}
  \item{gridserver}{
The name of the gridserver
}
  \item{port}{
The port associated with the gridserver
}

  \item{name, owner}{
The project name and owner (NOT optional)
}

\item{refmosaic}{
  The path to a reference mosaic which will be used to intitialize mosaic gridservers.  If supplied then mosaicinit will be 
  called prior to reading and writing. This is required if \code{as.mosaic} is \code{TRUE}.  You will also need to have 
  the required servers specified in gridservers.txt within the anthill directory.  See \code{\link[anthill]{readgridservers}}.
}

\item{extra.args}{
  (optional) a named list containing extra arguments to be passed to \code{FUN}
}

\item{...}{Arguments to be passed on to \code{\link[anthill]{launch.project}}} such as \code{priority}, \code{maxthreads}, and \code{onerror}.
}

\value{
These functions returns nothing they are called for their side effects:  \code{tilefun} writes out function output to a tile in \code{outgrid}; and \code{gridfun} and \code{pgridfun} write their output to a new grid.
}
\author{
Ethan Plunkett
}
\note{

\code{grids} and \code{vars} should be of the same length.  If \code{FUN} is a list than \code{outgrid} should be a vector of output grids of the same length.  

\code{\link{tileinit}} should be called prior to the \code{tilepredict} function.

\pkg{anthill} should be loaded and \code{\link[anthill]{config}} should be called prior to calling \code{pgridfun}.

The data is read and formatted as a list of vectors with names dictated by \code{vars}.  Then the function is called with those named variables.  If your function requires different argument names it will not work.
}


\seealso{
\code{\link{gridinit}} must be called prior to both functions and \code{\link{tileinit}} must be called prior to \code{tilefun}. 



\code{\link{tilepredict}}, \code{\link{gridpredict}}, \code{\link{pgridpredict}} are similar but call predict with a fitted model.

}
\examples{


  ### Setup a temporary directory with a sample grid in it 
  # Get path to example dem grid (included with package)
  require(gridio)
  datapath <- system.file("exampledata", package="gridio")  
  datapath <- shortPathName(datapath)
  datapath <- paste(datapath, "/.", sep="")
  
  # Make a temporary directory to write new grids to
  dir <- tempdir() # get temporary directory
  dir.create(dir)
  
  # Copy example data into temporary directory
  file.copy(datapath, dir, recursive=TRUE)
  # system(paste("open", dir)) # if you want to look at contents of temporary directory
  ###  Done setting up directory
  
  cleanup() # disconnect from any prior gridservers 
  gridinit() # initialize local gridserver with base path in temporary dir
  
  # setup paths
  dir <- gsub("\\\\\\\\", "/", dir)
  dem <- paste(dir, "/dem", sep="")
  gradientpath <- paste(dir, "/gradient", sep="")
  res1 <- paste(dir, "/res", sep="")
  res2 <- paste(dir, "/res2", sep="")
  
  
  setwindow(dem) # setwindow to dem file
  tileinit(150, 0)  # initialize tilescheme 
  # NOTE this you probably want 1000 to 3000 as the tilesize NOT 150
  #      this is a trivial example
  
  ### Make up some sample data
  g <- readgrid(dem)
  gradient <- seq(20, 100, length.out=g$nrow)
  g$m[,] <- gradient 
  g$m <- g$m + rnorm(g$nrow*g$ncol)
  writegrid(g, gradientpath)
  
  
  # Make functions to apply to the grids objects
  f1 <- function(a, b) a + b
  f2 <- function(a, b) a * b
  
  # call tilefun (one tile only)
  tileinit(150, 0)  # 2000, 0 would be more efficient
  tilefun(FUN=f1, grids=c(dem, gradientpath), vars=c("a", "b"), 
              outgrid=res1, tilenumber=1)
  r1 <- readgrid(res1)
  plot(r1)  # first tile only
  
  # And gridpredict (whole grid in tiles)
  gridfun(FUN=f1, grids=c(dem, gradientpath), vars=c("a", "b"),
              outgrid=res1, tilesize=150, overwrite=TRUE)  # do not use 150 as your tilesize!
  r1 <- readgrid(res1)
  plot(r1)
  
  
  # Call with mutliple function objects
  gridfun(FUN=list(f1, f2), grids=c(dem, gradientpath), vars=c("a", "b"), 
              outgrid=c(res1, res2), tilesize=1000, overwrite=TRUE)
  r2 <- readgrid(res2)
  plot(r2)
  
  
  \dontrun{ 
  
  # Make sure anthill is loaded, config() has been called and a gridserver 
  # has been launched before running the this.
  server <- computer.name() # or the name of the server
  port <- 3333  
  
  tilefun.args <- list(FUN=f1, grids=c(dem, gradientpath), 
                           vars=c("a", "b"), outgrid=res1) 
  # note this is the same as used with tilefun except for the tile argument
  
  pgridfun(tilefun.args=tilefun.args, gridserver=server, 
               port=port, name="pgridfuntest", owner="test",
               refgrid=tilefun.args$grids[1], tilesize=100, 
               maxthreads=3)  # Do not use 100 as your tilesize!
               
  info()
   } % end dontrun.       
  
  # Cleanup
  gridkill( c(dem, gradientpath, res1, res2))
  cleanup()
  
  
}
\keyword{ spatial }

