\name{gridsplit and gridstitch}
\alias{gridsplit}
\alias{gridstitch}

\title{
functions to break a grid into subgrids and assemble subgrids back into a single grid.
}
\description{
These functions allow a grid to be broken up into components and stitched back together.  The components
can be regularly tiled (with or without overlap) or can be irregularly tiled and specified with a polygon shapefile.
The intent is to use this to facilitate processing on very large grids by breaking them into smaller, more manageble grids. 
}
\usage{
gridsplit(complete, parts, rowsize, buffer, colsize = rowsize, index, id.field, 
          limit.extent = TRUE, tilesize = 2000, server = NULL, port = NULL)

gridstitch(parts, complete, buffer, ref, index, id.field, tilesize=2000, server = NULL, 
            port = NULL, add = FALSE, transparent, overwrite = FALSE, core.only = TRUE)
}

\arguments{
  \item{complete}{the path to the whole grid}
  \item{parts}{the path to the component grids should include "[tile]" which is a placeholder for the 
tilenumber or id of each subgrid.  If it doesn't include "[tile]" the tile number or id will be appended
}
  \item{rowsize, colsize}{ if using regular tiling these set the number of rows and columns 
  (cells not map units) that make up the core of each subgrid.
}
  \item{buffer}{ sets the distance (in cells) beyond the tile core 
  that will also be included in each subgrid.

}
  \item{index}{ the path to a polygon shapefile specifying the extent of the core of each subgrid. 
Using the \code{index} argument supersedes the \code{rowsize} and \code{colsize} arguments and indicates 
irregular tiling. 
}
  \item{id.field}{ this specifies the column name in the index shapefile that contains the ids of each 
  polygon (it's only used with irregular tiling.)  If omitted the rank order (after some filtering) 
  will be used as the id. 

}
  \item{limit.extent}{ this is currently only relevant to the index based tiling.  If TRUE the 
  subgrids will never extend beyond the original extent of the complete grid.  If FALSE they will extend by the buffer        distance.  Since the area beyond the original extent will always be NA the only reason to set this to TRUE is if the stitching will be additive or (perhaps) transparent and sparse; and the processing done prior to stitching will overwrite the NA's in the buffer with data. 
}
  \item{tilesize}{ This specifies the tilesize used for the individual read and write operations (not the size of each subgrid).  The default (2000) is reasonable but depending on the situation larger or smaller values might be more efficient.  The end result should not change at all with the tilesize.
}
\item{ref}{The reference extent that the complete grid should match.  Can be either the path to a grid on disk, a grid object in memory, or a list containing the elements xll, yll, nrow, ncol, cellsize}
  \item{server, port}{ Specifies the gridserver and port to use when interacting with the complete grid.  The component  grids will all be accessed in local mode.  If not specified everything will be done in local mode.
}

\item{add}{ if TRUE as the grids are stitched back together they are added to the complete grid}
  
  \item{transparent}{ if TRUE the grids are written transparently back into the original grid (NA's aren't written). 
  This defaults to TRUE for irregularly tiled stitching and FALSE for regularly tiled stitching. } 
  \item{overwrite}{if TRUE overwrite the output grid.  Currently this is only implemented for stitching. Splitting will overwrite output grids regardless of how this argument is set.}
  \item{core.only}{Only relevant to index based stitching; TRUE indicates that cells outside of the core (the original polygon) should be overwritten with NA prior to writing. If transparent is also TRUE (the default for index based tiling) then only the core of each tile will be written.  If FALSE then the entire subgrid will be written.  Since the subgrids are rectangular and cover the entirely of each polygon (possibly with a buffer) they will almost certainly overlap so FALSE is unlikely to be useful unless add=TRUE.  
  
  }

}

\details{

There are two different mechanisms tiled and indexed splitting; the presence of the index argument in the call determines which is used.

With tiled splitting each subgrid is a regularly sized and spaced rectangle in the landscape plus a fixed buffer.  Stitching simply scans the directory for grids who's names suggest they are component grids and then copies the core of each component grid into the output grid.  Thus the tiled stitching is less robust to missing files (it won't notice) or files with inappropriate extents (extents different then those created by the splitting process). Also, for logistic reasons [tile] should always be part of the grid name and not earlier in the path when using tiled splitting and stitching.

Indexed splitting uses a polygon shapefile to define the core of each subgrid. Each subgrid is then created with the minimal rectangular extent that contains the possibly corresponding polygon buffered by the buffer distance. Stitching back together by default uses only the cells from each subgrid that are within the original index polygon.  With indexed splitting [tile] can be anywhere in the path.

The buffer argument is a little awkward in \code{gridstitch}. It's only used for regular tiling where it indicates how many cells should be removed from the outer edge of each component grid prior to writing into the complete grid. Thus for the most common situation in which the user wants to stitch together the core of each grid, buffer should be set to the same value as was used with gridsplit.  To include the core and the buffer (presumably with add=TRUE) the buffer argument should be set to 0.  Other values will trim varying amounts before writing but I can't see any reason why they would be useful.

For index based tiling there is no buffering but the algorithm has two options:  if core.only is TRUE it will copy the core (cells inside the polygon) or the entire grid by first reading the grid, then overwriting NA outside the core and then writing the result into the output grid with transparent=TRUE (the default for index based tiling.) If core only is FALSE the area outside the polygon won't be overwritten with NA and thus the whole grid will be copied which is only likely to be useful if the add argument is TRUE. 


}
\value{
These functions create grids on disk but return nothing.
}
\author{
Ethan Plunkett
}

\seealso{
\code{\link{tileinit}} and \code{\link{subtileinit}} allow processing a grid in tiles without cutting it 
up into component grids.  
}
\examples{

  ### Setup a temporary directory with a sample grid in it 
  # Get path to example dem grid (included with package)
  require(gridio)
  datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
  datapath <- shortPathName(datapath)
  datapath <- paste(datapath, "/.", sep="")
  
  # Make a temporary directory to write new grids to
  dir <- tempdir() # get temporary directory
  dir <- gsub("\\\\\\\\", "/", dir)
  
  # Copy example data into temporary directory
  file.copy(datapath, dir, recursive=TRUE)
  
  cleanup() # disconnect from any prior gridservers 
  
  # Setup paths
  original <- paste(dir, "/dem", sep="")
  tiledparts <- paste(dir, "/tiledT[tile]", sep="")
  indexedparts <- paste(dir, "/indexedT[tile]", sep="")
  tiledstitch <- paste(dir, "/dem_tiled", sep="")
  indexedstitch <- paste(dir, "/dem_indexed", sep="") 
  

  index <- paste(dir, "/towns/towns.shp", sep="")
  id.field <- "TOWN_ID"
  
  buffer <- 10
  overwrite= TRUE
  
  # Verify that splitting and stitching back together produces a grid identical to the original
  
  # Note in these examples I wanted tileing to take place despite the small example grid so I set
  #  tilesize to be unreasonably small.  Don't do this when using the function.  tilesize=2000 
  #  (the default) or 3000 is reasonable. 
  
  
  # Split based on tiles
  gridsplit(complete=original, parts=tiledparts, rowsize=150, tilesize=50, buffer=buffer)
  
  # Split based on index shapefile
  gridsplit(complete=original, parts=indexedparts,index=index, buffer=buffer, 
            id.field = id.field, tilesize=100)
  
  # Stitch based on tiles
  gridstitch(parts=tiledparts, complete=tiledstitch,ref=original, buffer=buffer, 
             overwrite=overwrite, tilesize=75)
  
  # stitch based on index
  gridstitch(parts=indexedparts, complete=indexedstitch,ref=original,index=index, 
             id.field=id.field, overwrite=TRUE, tilesize=75)
  
  cleanup()
  gridinit()
  setwindow(original)
  g1 <- readgrid(original)
  g2 <- readgrid(tiledstitch)  
  g3 <- readgrid(indexedstitch)
  
  
  stopifnot(all.equal(g1, g2))
  stopifnot(all.equal(g1, g3))
  
   \dontshow{
  
  # Verify that the right data is coming from each subgrid (for index based subgrids)
  # by making copies of the subgrids in which all the cells are populated with the id.field
  # and then stitching those subgrids back together.
  
  # Build the index and make copies of subgrids populated with their ids
  a <- buildsubgridindex(index=index, window=griddescribe(original), id.field=id.field)
  ids <- a$ids
  ind <- a$ind
  paths <- character(length(ids))
  for(i in 1:length(a$ids)) paths[i] <- gsub("[tile]", ids[i], x=indexedparts, fixed=TRUE)
  outpaths <- paste(dir, "/idT", ids, sep="")
  for(i in 1:length(paths)){    
    cleanup()
    gridinit()
    setwindow(paths[i])
    g <- readgrid(paths[i])
    g$m[, ] <- ids[i]
    writegrid(g, path=outpaths[i], as.integer=TRUE)
  }
  parts <- paste(dir, "/idT[tile]", sep="") 
  complete <- paste(dir, "/stitchedids", sep="")
  
  # Stitch them together
  gridstitch(parts=parts, complete=complete, index=index, id.field=id.field, ref=original)
    
  cleanup()
  gridinit()
  setwindow(original)
  
  g4 <- readgrid(complete)
  # plot(g4, use.old=FALSE)
  # plot(a$ind, border="red", add=TRUE)
  
  # Now make the same directly from the polygon index
  sp <- as.SpatialPointsDataFrame(g1)
  proj4string(sp) <- proj4string(ind)
   o <- over(sp, ind)
  g5 <- g1
  g5$m[, ] <- o[[id.field]]
  plot(g5)
  stopifnot(all.equal(g4, g5))
  
  # end don't show:
  } 
  
  # Clear temporary directory
  if(file.exists(dir)) unlink(dir, recursive=TRUE)
  

}
\keyword{ spatial}

