\name{makemosaic}
\alias{makemosaic}
\alias{makemosaicpy}
\alias{batchmosaic}
\title{
functions to break up a standard grid into a mosaic
}
\description{
These function converts a standard grid into a mosaic.  Make mosaic depends on gridio only but does not work on very large grids.  makemosaicpy relies on ESRI python geoprocessing and can work with ESRI's multitiled grid format (used with big grids) \code{batchmosaic} calls \code{makemosaicpy} repeatedly.
}
\usage{
makemosaic(source, mosaic, rowsize, colsize = rowsize, panemap, 
           reference, tilesize = 2000, server = NULL, port = NULL)
makemosaicpy(source, mosaic, rowsize,  colsize=rowsize, referencemosaic, panemap, 
             python.path="C:/Python26/ArcGIS10.0/python.exe")
batchmosaic(x, referencemosaic)
}
\arguments{
  \item{source}{
The path to an existing grid.
}
  \item{mosaic}{
The path to the mosaic that will be created.
}
  \item{rowsize, colsize}{
The number of rows and columns of cells in each pane of the mosaic.
}
  \item{panemap}{
(optional) a logical vector indicating which panes have data.  Use \code{\link{mosaicinfo}} to get the panemap from another mosaic. This can also be inputed as a string of "0" and "1" in a character scalar.
}
  \item{reference}{
(optional) - a reference grid used to populate panemap - panes with data in the reference grid 
will be included in the mosaic. 
}
\item{referencemosaic}{
  (optional) the path to a reference mosaic. It will be used to populate the panemap, rowsize, and colsize.
}

  \item{tilesize}{
the size of the tiles that are read and written in each operation. This is independent of the
colsize and rowsize that are used to define the mosaic panes on disk.
}
  \item{server, port}{
(optional) The gridserver and port used for interacting with the whole landscape. Leave blank for local mode. The panes are always written in local mode.
}
\item{python.path}{
This is the path to the python executable on the system. The default works with ArcGIS 10.
}
\item{x}{ a vector of grids from which mosaics should be made.
}
}

\details{

\code{makemosaic}

 If neither panemap nor reference are supplied, the panemap is generated from the source grid.
 If server is undefined than reads will be done in local mode.  (Writes are always local).
 Note to read or write to mosaics you must first call \code{\link{mosaicinit}}; however, it is not necessary to call \code{\link{mosaicinit}} prior to \code{makemosaic}.  
 
  \code{makemosaic} creates a template gridservers.txt file inside the mosaic directory. If you define ports in this file and add it to the anthill directory (or append it to an existing gridservers.txt) then anthill will be aware of the gridservers and \code{\link[anthill]{readgridservers}}, \code{\link[anthill]{launch.gridservers}}, and \code{\link[anthill]{relaunch.gridservers}} will all work. 

\code{ makemosaicpy}

With makemosaicpy the panemap comes either from the referencemosaic or is derived from the source grid.  
It relies on ArcGIS python geoprocessor which is a little buggy.  It is likely to fail if 
either \code{source} or \code{mosaic} are on a network drive. Error capturing from the python script back into R is
not great. If the python script fails it may be helpful to run it manually for debugging. Look for tile.py inside the directory specified by \code{mosaic}.

\code{ batchmosaic }
This is a wrapper for \code{makemosaicpy}.  Error reporting is suppressed during execution and instead a summary of any errors is returned after attempting to create all the mosaics.

}
\value{
A mosaic grid is created on disk from the source grid(s).  \code{batchmosaic} returns a dataframe listing any errors encountered.}
\author{
Ethan Plunkett
}
\seealso{
\code{\link{launchbatchmosaic}} uses Anthill to make mosaics on another machine, possibly in parallel.
\code{\link{newmosaic}}, \code{\link{mosaicinit}}, \code{\link{stitchpy}}
}
\examples{

datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
datapath <- shortPathName(datapath)
datapath <- paste(datapath, "/.", sep="")

dir <- tempdir() # get temporary directory

# Clear it out
if(file.exists(dir)) unlink(dir, recursive=TRUE)
if(!file.exists(dir)) dir.create(dir)
list.files(dir)

# Copy example data into temporary directory
file.copy(datapath, dir, recursive=TRUE)

# Setup paths
source <- paste(dir, "/dem", sep="")
mosaic <- paste(dir, "/demmosaic", sep="")
mosaic2 <- paste(dir, "/demmosaic2", sep="")
mosaic3 <- paste(dir, "/demmosaic2", sep="")
mask <- paste(dir, "/mask", sep="")

# Make mask grid  
cleanup()
gridinit()
setwindow(source)
g <- readgrid(source)
g$m[,] <- NA
g$m[row(g$m) + col(g$m) > 300] <- 1 
plot(g, use.old=TRUE)  
writegrid(g, path=mask)

# Make mosaics
if(ismosaic(mosaic)) gridkill(mosaic)
makemosaic(source, mosaic, 200)

if(ismosaic(mosaic2)) gridkill(mosaic2)
makemosaic(source, mosaic2, reference=mask,  200)

panemap <- '0001' # could also use a logical vector
if(ismosaic(mosaic3)) gridkill(mosaic3) # d
makemosaic(source, mosaic3, panemap=panemap,  200)


gridinit()
setwindow(source)
mosaicinit(reference=mosaic) # setup for reading and writing to mosaics in local mode
g <- readgrid(mosaic)
g2 <-readgrid(source)

if(!isTRUE(all.equal(g, g2))) stop("Mosaic grid and standard grid aren't the same!!")

}
\keyword{ spatial}
