\name{readgrid}
\alias{readgrid}
\alias{writegrid}
\alias{writegrid.grid}

\title{
Functions to read and write entire grids
}
\description{
These functions read and write entire grids.
}
\usage{
readgrid(path, as.matrix = FALSE, na.value = -9999, cacheok = FALSE)
writegrid(x, path, as.integer = FALSE, na.value = -9999)
}
\arguments{
  \item{x}{ the grid to be written} 
  \item{path}{
the name and path to the grid to read.  If it is a relative path is should be relative to the basepath set by \code{\link{gridinit}}.
}
  \item{as.matrix}{
if \code{TRUE} than \code{readgrid} returns a matrix instead of a \code{\link{grid}} object.
}
  \item{as.integer}{
if \code{TRUE} than \code{writegrid} creates an integer grid instead of the default real grid.
}
  \item{na.value}{
this is the value used to transmit NA cells to the underlying C code. The only reason to change it from the default is if the default value (-9999) occurs in the grid.
}
 \item{cacheok}{if TRUE than this grid is eligible for caching (if caching is configured). See \code{\link{cacheinit}}.
}
}

\value{
\code{writegrid} returns nothing.  \code{readgrid} returns either a grid or a matrix; which depends on the argument \code{as.matrix}.
}
\author{
Ethan Plunkett
}

\seealso{
\code{\link{readblock}}, \code{\link{writeblock}}, \code{\link{readtile}}, \code{\link{writetile}}
}
\examples{

### Setup a temporary directory with a sample grid in it 
# Get path to example dem grid (included with package)
require(gridio)
datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
datapath <- shortPathName(datapath)
datapath <- paste(datapath, "/.", sep="")

# Make a temporary directory to write new grids to
dir <- tempdir() # get temporary directory
if(!file.exists(dir)) dir.create(dir)

# Copy example data into temporary directory
file.copy(datapath, dir, recursive=TRUE)
# system(paste("open", dir)) # if you want to look at contents of temporary directory
###  Done setting up directory



# Setup paths
dem <- paste(dir, "/dem", sep="") # path to dem file.
demcopy <- paste(dir, "/demcopy", sep="")
demcopy2 <- paste(dir, "/demcopy2", sep="")
demcopy3 <- paste(dir, "/demcopy3", sep="")
ec <- paste(dir, "/ec", sep="")
ec2 <- paste(dir, "/ec2", sep="")

# initialize
cleanup() # disconnect from any prior gridservers 
gridinit()
setwindow(dem) # setwindow to dem file

# Read a grid
g <- readgrid(dem)
plot(g)  

# write it out to a new grid
writegrid(g, demcopy, as.integer=FALSE)

# Read the copy in 
f <- readgrid(demcopy)

# Test to see if the copy is identical to the original
if(!isTRUE(all.equal(f$m, g$m))) stop("grid read write failed to produce identical grid")
if(!coincide(f, g)) stop("grid read write failed to produce matching grid alignments")

# Now make a copy using the readtile and writetile meachanism
# Note this tile size is inefficient 2000 and up is recommended  
#  this small size is being used because the test grid is tiny and I want to break it up into 
#  multiple tiles
tileinit(150, 0)

for(i in 1:ntiles()){
 settile(i)
 tile <- readtile(dem)
 writetile(tile, demcopy2, as.integer=FALSE)
}

h <- readgrid(demcopy2)

# Test to see if the copy is identical to the original
if(!isTRUE(all.equal(h$m, g$m))) stop("grid read write failed to produce identical grid")
if(!coincide(h, g)) stop("grid read write failed to produce matching grid alignments")


# Repeat test but with integer grid:
g$m[,] <- round(g$m/50)*50  # round to nearest 50 meters
plot(g)

writegrid(g, ec, as.integer=TRUE)  # elevation class

if(gridtype(ec)!= "integer") stop("Write grid failed to write the correct type")

for(i in 1:ntiles()){
  settile(i)
  tile <- readtile(ec)
  writetile(tile, ec2, as.integer=TRUE)
}

e <- readgrid(ec2)
plot(e)

# Test to see if the copy is identical to the original
if(!isTRUE(all.equal(e$m, g$m))) stop("grid read write failed to produce identical grid")
if(!coincide(e, g)) stop("grid read write failed to produce matching grid alignments")


# Cleanup
gridkill(c(dem, demcopy, demcopy2, demcopy3, ec, ec2))
cleanup()

}
\keyword{ spatial }
