


\name{samplegrid}
\alias{samplegrid}
\title{
Function to sample a grid at specified locations
}
\description{
This function returns a vector of values from the grid for each location specified.
}
\usage{
samplegrid(grid.path, loc, proj, tilesize = 2000, checkproj = TRUE, init = TRUE, cacheok = TRUE)
}
\arguments{
  \item{grid.path}{
the path to the grid to be sampled
}
  \item{loc}{
  a spatial points data frame. It can be created from a dataframe with the \code{\link{coordinates}}
  function. If checkproj is \code{TRUE} than \code{loc} should have a projection defined as well 
  which can be done with the \code{\link{proj4string}} function.  
}
  \item{proj}{
  the projection associated with the grid on disk - this argument is not necessary if the projection of the grid is defined on disk) 
}
  \item{tilesize}{
the tilesize with which to process the grid
}
  \item{checkproj}{
if TRUE the projections are checked and reprojection is managed by the function if FALSE it is assumed that the coordinate systems match
}
  \item{init}{
   if TRUE the function calls \code{\link{gridinit}} in local mode on the target grid.  Otherwise it assumes that \code{\link{gridinit}} and \code{\link{setwindow}} have already been called (either in local or server mode)
}

\item{cacheok}{ passed to \code{\link{readtile}} TRUE allows local caching of grids when caching is on.
}
}
\details{
This function samples the grid at the points specified in loc by reading the grid in tiles.  Larger
\code{tilesize} is generally more efficient but required more memory. If you know your points and grid are in the same projection set \code{checkproj} to \code{FALSE} otherwise loc will need to have a projection defined and either the grid on disk will need to have a projection defined or you will need to specify it with \code{proj}.
}
\value{
a vector of grid values - one for each point in loc is returned
}
\author{
Ethan Plunkett
}
\seealso{

Several functions do more or less the complement of samplegrid:
\code{\link{pointstogrid}} creates a grid from a set of points.  \code{\link{gaussianinterpolate}} creates a smoothed grid from a set of points.  \code{\link{densityweight}} counts how many points fall in each cell of a grid and an associated weight for each point.
}
\examples{


### Setup a temporary directory with a sample grid in it 
# Get path to example dem grid (included with package)
require(gridio)
datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
datapath <- shortPathName(datapath)
datapath <- paste(datapath, "/.", sep="")

# Make a temporary directory to write new grids to
dir <- tempdir() # get temporary directory
if(!file.exists(dir)) dir.create(dir)

# Copy example data into temporary directory
file.copy(datapath, dir, recursive=TRUE)
# system(paste("open", dir)) # if you want to look at contents of temporary directory
###  Done setting up directory



# Setup paths
dem <- paste(dir, "/dem", sep="") # path to dem file.
demcopy <- paste(dir, "/demcopy", sep="")
demcopy2 <- paste(dir, "/demcopy2", sep="")
demcopy3 <- paste(dir, "/demcopy3", sep="")
ec <- paste(dir, "/ec", sep="")
ec2 <- paste(dir, "/ec2", sep="")

# initialize
cleanup() # disconnect from any prior gridservers 
gridinit()
setwindow(dem) # setwindow to dem file

# Make random points accross the landscape
n <- 50 
b <- getbounds(getwindow())
pts <- data.frame(x=runif(n, b[1, 1], b[1, 2]), y=runif(n, b[2, 1], b[2, 2]))

# Convert to spatial points data.frame
coordinates(pts) <- c("x", "y")
str(pts)
sample <- samplegrid(grid.path=dem, loc=pts, checkproj=FALSE, tilesize=150)

# Plot the points with larger circles at higher elevations
cex <- 1 + 2 * (sample - min(sample, na.rm=TRUE))/ (max(sample, na.rm=TRUE) - min(sample, na.rm=TRUE))
g <- readgrid(dem)
plot(g)
points(pts, cex=cex, col="red")

gridkill(dem)
cleanup()

}
\keyword{spatial }
