\name{subtiles}
\alias{subtile}
\alias{subtiles}
\alias{subtileinit}
\alias{setsubtile}
\alias{stinit}
\alias{extractsubtile}
\alias{sti}
\alias{stri}
\alias{stci}
\alias{stid}
\alias{nsubtiles}
\alias{newtile}


\title{
functions to work with subtiles
}
\description{
These functions facilitate reading grids in tiles and processing but processing them in smaller
sub-tiles. Every cell falls into a single subtile and tile and every subtile falls entirely within
one tile. }
\usage{
subtileinit(rowsize, buffer, tilesize, nr, colsize = rowsize, nc = nr)
stinit()
setsubtile(subtile)
nsubtiles()
newtile(coreonly=FALSE)
sti(coreonly = FALSE)
stci(coreonly = FALSE)
stri(coreonly = FALSE)
extractsubtile(x, coreonly = FALSE, safe = FALSE)
stid()
}

\arguments{
 \item{rowsize, colsize}{
The dimension of each subtile (in cells.)
}
  \item{buffer}{
The buffer size in cells for both the tile and subtile schemes.  As tiles are processed they will 
include both the core of the tile or subtile and this many additional cells.  
}
  \item{tilesize}{
This the approximate dimension of the tiles in cells.  The exact dimensions will be whatever
integer multiple of \code{rowsize} and \code{colsize} falls closest to \code{tilesize}
}
  \item{nr, nc}{
This is an alternate mechanism for specifying tilesize. \code{nr} and \code{nc} are the number of
rows and columns of subtiles that make up each tile.
}
\item{subtile}{
A subtile number, an integer between 1 and \code{nsubtiles()}.
}

\item{x}{
a grid object that coincides with the current tile.  Typically it will be created by 
\code{\link{readtile}} or \code{\link{newtile}}.
}
  \item{coreonly}{ controls buffer inclusion in the subtile or tile. If \code{TRUE} only the core
 is used or included. If \code{FALSE} both the core and the buffer are used.
}
  \item{safe}{
  If \code{TRUE} the function checks to make sure \code{x} coincides with the current tile in the 
  tilescheme; this is safer and slower. 
}
}
\details{

\itemize{
\item{\code{subtileinit} initializes a subtile and tile scheme.}
\item{\code{stinit} reinitializes the tile specific parts of a subtile scheme.  It is called by 
subtileinit and settile and is primarily for internal use.}
\item{\code{setsubtile} sets the current subtile.}
\item{\code{nsubtiles} returns the number of subtiles in the current tile.}
\item{\code{newtile} creates an empty grid coinciding with the current tile and its buffer. }
\item{\code{sti} creates a matrix that can be used to select the subtiles cells from a 
tile sized grid's matrix.}
\item{\code{stci} and \code{stri} create vectors that select the subtiles rows and columns from a 
  tile sized matrix.}
\item{\code{extractsubtile} extracts a subtile-sized grid object from a tile-sized grid object.}
\item{\code{stid} returns a unique subtile id for each subtile in the subtile scheme. The id's are integers
representing the order the subtiles would be encountered in starting from the top left and progressing 
left to right and top to bottom using row dominant indexing.}
}

The workflow for processing a grid in subtiles will generally be:
\enumerate{
\item{  call \code{subtileinit} to set up the sub-tile and tile schemes}
\item { loop thorough tiles (1:\code{\link{ntiles}}) or process in parallel}
\item{ call \code{settile} }
\item{ use \code{newtile} to make an empty tile to store results }
\item{ use \code{\link{readtile}} to read in required tiles }
\item{ loop through subtiles (1:\code{nsubtiles()}) }
\item{ extract relevant data from the tiles read in step 5  with \code{extractsubtile}, \code{sti}, or 
\code{stri} and \code{stci} }
\item{ calculate stuff }
\item{  save results into the grid created in 4 with \code{\link{overlay}} }
\item{ end loop through subtiles }
\item{ save results with \code{\link{writetile}} }
\item{ end loop through tiles }
}

}
\value{
\code{subtileinit}, \code{stinit}, and \code{setsubtile} return nothing but make changes to the
subtile scheme.

\code{nsubtiles} returns an integer indicating the number of subtiles in the current tile.

\code{newtile} returns a grid that coincides with the current tile and is filled with \code{NA} cells.

\code{sti} return a logical matrix corresponding to the dimensions of the current tile with \code{TRUE} 
in cells that are part of the current subtile or its buffer.

\code{stri} and \code{stci} return vectors of the row and column numbers that the current subtile and
its buffer occupy within the current tile. 

\code{extractsubtile} returns a grid object with data from \code{x} for the current subtile.

\code{stid} returns the unique subtile ID of the current tile, an integer between 1 and the total 
number of subtiles in the subtile scheme.  The subtile id is different than the subtile number(used 
by \code{setsubtile} and \code{nsubtiles}) because the subtile numbers restart with every tile.

}
\author{
Ethan Plunkett
}
\note{

\code{sti},\code{stri}, \code{stri}, and \code{extractsubtile} all expect to work with either a 
matrix or tile that was created from the current tile; typically produced by \code{readtile} if they 
are used with an object that doesn't align with the current tile bad things will happen.  Setting 
\code{safe} to \code{TRUE} will force \code{extractsubtile} to check that assumption.  
}

\seealso{
\code{\link{tileinit}} for initializing a tile scheme without a subtile scheme.  \code{\link{overlay}} for 
overlaying data in a subtile grid into a tile grid. 
}
\examples{


### Setup a temporary directory with a sample grid in it 
# Get path to example dem grid (included with package)
require(gridio)
datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
datapath <- shortPathName(datapath)
datapath <- paste(datapath, "/.", sep="")

# Make a temporary directory to write new grids to
dir <- tempdir() # get temporary directory
if(!file.exists(dir)) dir.create(dir)

# Copy example data into temporary directory
file.copy(datapath, dir, recursive=TRUE)
# system(paste("open", dir)) # if you want to look at contents of temporary directory
###  Done setting up directory

# setup paths
dem <- paste(dir, "/dem", sep="") # path to dem file.
smoothed <- paste(dir, "/smoothed", sep="")

cleanup() # disconnect from any prior gridservers 
gridinit()
setwindow(dem) # setwindow to dem file

###  Example workflow for working in subtiles. This is a toy example that could be done trivially 
#      without using tiles or subtiles. Here we are going to compute a guassian smooth
#      of a dem on 50 by 50 cell subtiles that are aggregated into 100 by 100 cell tiles.
#      Generally for efficiency's sake you will want to use tilesizes of at least 2000 possibly more.
#      
####
      # 1. initilize subtile scheme
      
subtileinit(rowsize=75, buffer=20, tilesize=150)
plotextent()  # Visualize it (optional)

for(tile in 1:ntiles()){             # 2.  Loop through tiles
  settile(tile)                      # 3. Call set tile
  result <- newtile()                # 4. Make an empty tile-sized grid to store results
  input <- readtile(dem)           # 5. read in input data for tile  
  for(i in 1:nsubtiles()){           # 6. loop through subtiles 
    setsubtile(i)
    st <- extractsubtile(input)      # 7. extract subtiles from tile inputs 
      
    st <- gaussiansmooth(st, sd=30) # 8.  calculate stuff - in this case smooth the data 
       # 9. overlay the core of the subtile on result.  For some workflows you may use the
       #  entire subtile or the transparent or add arguments
    result <- overlay(st, result, coreonly=TRUE)
  }    # 10.  end loop through subtiles
       # 11. Save the results
  writetile(result, smoothed, as.integer=FALSE)    
}      # 12. end loop through tiles


#  Examine results and compare to smoothed performed in a single step
g <- readgrid(dem)  # orignal grid
r1 <- readgrid(smoothed)  # smoothed (in subtiles above)
r2 <- gaussiansmooth(g, sd=30)  # smooth in one step (much more efficient)

# Make a grid of differences
diff <- r1
diff$m[,] <- NA
diff$m <- r1$m - r2$m

# Verify that the grid produced by processing in subtiles is the same 
#   as that produced in one step.
#  Allow minor differences because one has been written to disk where it
#  is stored with lower precision.
if(!coincide(r1, r2)) stop("results don't coincide")
tolerance <- 1E-4
if(!isTRUE(all.equal(is.na(r1$m), is.na(r2$m)))) stop("Differences in NA's")
if(any(abs(diff$m)> tolerance, na.rm=TRUE)) stop("grids values are different")

gridkill(c(dem, smoothed))
cleanup()


}
\keyword{ spatial }

