\name{tilehist}
\alias{tilehist}
\title{
Function to assemble histogram data for an entire grid one tile at a time.
}
\description{
This function facilitates the plotting of histograms of grids that are too big to fit in memory. The histogram can be of all cells in a grid or can be calculated for every class of cell stored in a second grid.
}
\usage{
tilehist(data.path, class.path = NULL, tilesize = 2000, breaks = NULL, binsize = 10, ...)
}

\arguments{
  \item{data.path}{
The path to the grid containing the values from which the histogram will be made.
}
  \item{class.path}{
(Optional) A path to a grid containing classes. If this is specified then separate counts will be calculated for every class in this grid.
}
  \item{tilesize}{
The tilesize to use when constructing the counts. This should have no effect on the final outcome but will impact the speed of the calculation.
}
  \item{breaks}{
(Optional) a vector of break points between histogram cells. 
}
  \item{binsize}{
The bin size to be used if breaks is not specified. 
}
 \item{...}{
  Other arguments to \code{\link{hist}} that effect the counts returned such as \code{right} and \code{include.lowest}.
}
}
\details{
This function loops through all the tiles in the grid and assembles a vector of (binned) counts either for the entire grid or for each class in the grid specified by \code{class.path}.  See \code{\link{plot.tilehist}} for how to make plots from the resulting object.  The breaks in the histogram cells must be the same for each tile and must be determined before the first tile is processed and can either be specified by the caller via the \code{breaks} argument or calculated by the function based on the range of data in the grid and the \code{binsize} argument.  In the second case the bins are centered on multiples of \code{binsize} e.g. with a binsize of 10 and a minimum value in the grid of 0 the first three breaks will be -5, 5, and 15.

}
\value{ An object of class "tilehist" with these elements:
  \item{counts}{if \code{class.path} isn't \code{NULL} this will be a list of numeric vectors containing bin counts, with an item for each class in the class grid.  If \code{class.path} is \code{NULL} it will be a vector of counts for each bin.}
  \item{breaks }{a numeric vector of the breaks that define the bins of the histogram.}
  \item{mids}{a numeric vector of the midpoints of each cell in the histogram.}
  \item{classes}{The names of each class in \code{counts} this will only be included if \code{class.path} isn't \code{NULL}.}
}

\author{
Ethan Plunkett
}
\note{
 \code{\link{hist}} is called on each tile in the grid with plot=FALSE, and the breaks that are either specified or precalculated for the entire grid.  The counts element of the call is saved and added to a running tally.
}

\seealso{
\code{\link{plot.tilehist}} for plotting the resulting data, \code{\link{hist}} for other arguments that can be specified via \code{...}.
}
\examples{

### Setup a temporary directory with a sample grid in it 
# Get path to example dem grid (included with package)
require(gridio)
datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
datapath <- shortPathName(datapath)
datapath <- paste(datapath, "/.", sep="")
dir <- tempdir() # get temporary directory
dir.create(dir)
# Copy example data into temporary directory
file.copy(datapath, dir, recursive=TRUE)

# setup paths
dem <- paste(dir, "/dem", sep="")
groupspath <- paste(dir, "/groups", sep="")

cleanup() # disconnect from any prior gridservers 
gridinit() # initialize local gridserver
setwindow(dem) # setwindow to dem file

### Make up a grouping grid
g <- readgrid(dem)
groups <- g
groups$m[,] <- 1
groups$m[1:groups$nrow > 150,] <- 3
groups$m[,1:groups$ncol > 150] <- groups$m[,1:groups$ncol > 150] +1 
groups$m[g$m > 200] <- 5
writegrid(groups, groupspath, as.integer=TRUE)


# Note these are trivial example because the grid can easily fit in memory.  
# I'm using a excessively small tilesize so that there are multiple tiles but I 
# recommend using a tilesize of 2000 or more. I'm not sure where the optimum is.

# Call with a class (grouping) grid.
th <- tilehist(dem, groupspath, tilesize=100)
plot(g)  # the dem 
plot(groups) # the arbitrary grouping
plot(th, "1") # this is the group in the northwest of the area
plot(th, "5")  # this is the group made up of all the values greater than 200.
plot(th, "5" , freq=FALSE) # density plot of group 5

# call with no grouping grid (histogram of entire grid)
th <- tilehist(dem, tilesize=200) 
par(mfrow=c(2, 1))
d <- plot(th)
hist(g$m, breaks=d$breaks) # compare to hist of entire grid with same breaks
gridkill(c(dem, groupspath))  # delete grids

cleanup()
}
\keyword{ spatial }

