# Gaussian Smooth Testing

### Setup a temporary directory with a sample grid in it 
# Get path to example dem grid (included with package)
require(gridio)
datapath <- system.file("exampledata", package="gridio")  # grid path to example grid
datapath <- shortPathName(datapath)
datapath <- paste(datapath, "/.", sep="")

# Make a temporary directory to write new grids to
dir <- tempdir() # get temporary directory
if(!file.exists(dir)) dir.create(dir)

# Copy example data into temporary directory
file.copy(datapath, dir, recursive=TRUE)
# system(paste("open", dir)) # if you want to look at contents of temporary directory
###  Done setting up directory

cleanup() # disconnect from any prior gridservers 

gridinit() # initialize local gridserver with base path in temporary dir
grid <- paste(dir, "/dem", sep="")
setwindow(grid) # setwindow to dem file

gd <- griddescribe(grid)
n <- 200
sample.points <- data.frame(x=runif(n=n, min=gd$xll, max=gd$xll+ gd$ncol*gd$cellsize), 
                            y=runif(n=n, min=gd$yll, max=gd$yll+ gd$nrow*gd$cellsize))

sd <- 600
method <- 3
tilesize=200
max.p.na = 1
outgrid = paste("ds", sd,"_", method, sep="")
overwrite=TRUE
x <- grid
max.r <- sd*3
kernel.dim = 9

# smooth without sampling
for(method in 1:3)
  gaussiansmooth(grid, sd=sd,  tilesize = 200, max.p.na = 1, 
                  outgrid=paste(dir, "/ds", sd,"_", method, sep="") ,
                 overwrite=TRUE, method=method, use.old=FALSE)

g1 <- readgrid(paste(dir, "/ds", sd,"_", 1, sep=""))
g2 <- readgrid(paste(dir, "/ds", sd,"_", 2, sep=""))
g3 <- readgrid(paste(dir, "/ds", sd,"_", 3, sep=""))
g4 <- gaussiansmooth(readgrid(grid), sd=sd, max.p.na=1, use.old=TRUE)


range(g1$m - g2$m, na.rm=TRUE)
range(g1$m - g3$m, na.rm=TRUE)
range(g1$m - g4$m, na.rm = TRUE)

if(!isTRUE(all.equal(g1, g2))) stop("Mismatch")
if(!isTRUE(all.equal(g1, g3))) stop("Mismatch")


# diff <- g1
# diff$m <- g4$m - g1$1
# plot(diff)  # super trippy plot of
## interpolated rounding error badsed differences between the R and C++ 
## versions of the functions

sample.values <- matrix(NA, nrow=n, ncol=3)

# smooth with sampling
for(method in 1:3){
  sample.values[ ,method] <- gaussiansmooth(grid, sd=sd, 
                                            tilesize = 200,
                                            max.p.na = 1, 
                                            outgrid=paste("ds", sd,"_", method, sep="") , 
                                            overwrite=TRUE, 
                                            method=method, 
                                            sample.points=sample.points, 
                                            use.old = FALSE)
}
  
  max(apply(sample.values, 1, function(x) max(x)-min(x)), na.rm=TRUE) # should be zero or close to it

g1 <- readgrid(paste(dir, "/ds", sd,"_", 1, sep=""))
g2 <- readgrid(paste(dir, "/ds", sd,"_", 2, sep=""))
g3 <- readgrid(paste(dir, "/ds", sd,"_", 3, sep=""))


if(!isTRUE(all.equal(g1, g2))) stop("Mismatch")
if(!isTRUE(all.equal(g1, g3))) stop("Mismatch")


### Test with weights:
# make a copy of the dem with catagorical data
g <- readgrid(grid)
breaks <- quantile(g$m, probs=seq(0, 1, .25), na.rm=TRUE)
for(i in 1:length(breaks)){
  if(i < length(breaks))
    g$m[g$m >= breaks[i] & g$m < breaks[i+1]] <- i
  else 
    g$m[g$m >= breaks[i]] <- i
}
grid <- paste(dir, "/demclass", sep="")
writegrid(g, grid)
g <- readgrid(grid)

# Make weights table
weights <- data.frame(value=1:4, weight=c(1, 2, 4, 8))

sample.values <- matrix(NA, nrow=n, ncol=3)
for(method in 1:3){
  sample.values[ ,method] <- gaussiansmooth(grid, sd=sd,  tilesize = 200, max.p.na = 1, 
                                outgrid=paste(dir, "/ds", sd,"_", method, sep="") , overwrite=TRUE, 
                                method=method, sample.points=sample.points, weights=weights)
}
  

 diffs <- apply(sample.values, 1, function(x) max(x)-min(x))
  mean(diffs) # should be zero or close to it
  max(diffs)   # should be zero or close to it
  

g1 <- readgrid(paste(dir, "/ds", sd,"_", 1, sep=""))
g2 <- readgrid(paste(dir, "/ds", sd,"_", 2, sep=""))
g3 <- readgrid(paste(dir, "/ds", sd		,"_", 3, sep=""))

if(!isTRUE(all.equal(g1, g2))) stop("Mismatch")
if(!isTRUE(all.equal(g1, g3))) stop("Mismatch")



### Test with weights and no.match and na.value arguments
grid <- paste(dir, "/demclass", sep="")
g <- readgrid(grid)

# Make weights table
weights <- data.frame(value=1:3, weight=c(1, 2, 4))

for(method in 1:3){
  gaussiansmooth(grid, sd=sd,  tilesize = 200, max.p.na = 1, 
                  outgrid=paste(dir, "/ds", sd,"_", method, sep="") , overwrite=TRUE, method=method, 
                  sample.points=sample.points, weights=weights, no.match=8)
}

g1 <- readgrid(paste(dir, "/ds", sd,"_", 1, sep=""))
g2 <- readgrid(paste(dir, "/ds", sd,"_", 2, sep=""))
g3 <- readgrid(paste(dir, "/ds", sd  	,"_", 3, sep=""))

if(!isTRUE(all.equal(g1, g2))) stop("Mismatch")
if(!isTRUE(all.equal(g1, g3))) stop("Mismatch")

  
gridkill(x=paste(dir, "/", c("dem","demclass", "ds600_1", "ds600_2", "ds600_3"), sep=""))
gridlist(dir)
cleanup()



